#ifndef __ANALYSABLE_H__
#define __ANALYSABLE_H__

#include <list>
#include <vector>
#include <string>

#include <signature.h>


//! Analysierbares Objekt.
/*!
 * Alle Objekte einer Simulation die Messwerte liefern und diese zur Auswertung
 * bereit stellen wollen, implementieren diese Klasse. Ein zugehoeriger Analyser
 * stoesst die Messung an.
 * 
 * @see Analyser
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Analysable
{
public:

	//! Die Arten von Messungen.
	enum TypeInfo 
	{ 
		eNoObservation=0, 
		eDelay,
		eLatency,
		eQueue, 
		eSThroughput, 
		eTThroughput, 	
		ePrioriDelay,
		ePrioriLatency,
		ePrioriTThroughput, 
		ePacketLoss, 
		eLoad
	};

	//! Informationen zu Messung.
	struct Description 
	{
		//! Art der Messung.
		TypeInfo type;
		//! Messvariable fuer Auswertung.
		unsigned int variable;
		//! Nachrichtenprioritaet.
		unsigned int priority;
	};

	//! Datum fuer beliebige Messung.
	struct Data : public Signature
	{
		//! Nachrichtenprioritaet.
		unsigned int priority;
		//! Messwert.
		double value;
	};

	//! (Virtueller) Destruktor.
	virtual ~Analysable() {}	
	
	//! Bestimmung eines Messwertes.
	/*! 
	 * Ein Messwert fuer eine Messung wird bestimmt und zurueckgeliefert.
	 *
	 * @param type Die Art der Messung.
	 * @return Messwert fuer spezifizierte Messung.
	 */
	virtual double Measure(TypeInfo type) = 0;
	
	//! Durchfuehrung einer Messung.
	/*! 
	 * Es wird ein Messwert bestimmt und der Auswertung uebergeben. Die
	 * Auswertung erfolgt derzeit durch Akaroa2.
	 *
	 * @param description Art der Messung und Variable fuer Auswertung.
	 */
	virtual void takeMeasure(const Description& description);
	
	//! Durchfuehrung einer Messung erzwingen.
	/*! 
	 * Anstatt einen neuen Messwert zu bestimmen, wird das zweite Argument direkt
	 * der Auswertung uebergeben.
	 * 
	 * @param description Art der Messung und Variable fuer Auswertung.
	 * @param value Wert fuer Auswertung.
	 */
	void forceMeasure(const Description& description, double value);
};

#endif
