#ifndef __BUFFER_H__
#define __BUFFER_H__

#include <triggernode.h>

class BufferQueue;

//! Die Puffer eines Verbindungsnetzes.
/*!
 * Diese Klasse beschreibt die Puffer eines Verbindungsnetzes. Puffer 
 * besitzen eine Queue, die Flits von Paketen aufnimmt und ueber ein
 * Senderegister an nachfolgende Netzkomponenten weitervermittelt. Das
 * Senderegister sorgt dafuer, dass im Falle von Multicast ein Flit
 * erst dann aus der Queue entfernt wird, wenn es von allen Nachfolgern
 * empfangen wurde.
 *
 * @see BufferQueue
 * @see RegisterContainer
 * 
 * Puffer sind Knoten in einem Graphen mit genau einem Vor- und Nachfolger,
 * die ueber (gerichtete) Leitungen verbunden sind. Weiterhin besitzen 
 * Puffer Bitmasken, die alle in Senderichtung erreichbaren Ziele maskieren.
 * Zur Realisierung von Paket- bzw. Flitbewegungen in einem Netztakt werden
 * Puffer von einer Coroutine zm Senden oder Empfangen aufgefordert.
 *
 * @see Component
 * @see Node
 * @see TriggerNode
 * @see PacketForwarding
 * 
 * An Puffern koennen Messungen zur Laenge der Queue (Flits), sowie zu 
 * Verzoegungs- und Latenzzeiten (Pakete) durchgefuehrt werden.
 * 
 * @see Analysable
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Buffer : public TriggerNode
{

public:
	
	//! Konstruktor.
	/*!
	 * Es erfolgen lediglich notwendige Initialisierungen der Klassenattribute.
	 * In seiner Eigenschaft als XMLElement kann ein zugehoeriges DOMElement
	 * fuer den Puffer spezifiziert werden. Der Abschluss der Initialisierung
	 * eines Puffers erfolgt erst durch Aufruf der Methode init.
	 *
	 * @see XMLElement
	 *
	 * @param scope Zugehoeriges DOMElement mit Beschreibung dieses Puffers.
	 */
	Buffer(const DOMElement* scope=NULL) : BQ(0) 
	{
		setScope(scope);
		maeander = false;
	}
        	
	//! Implementierung der abstrakten Basisklasse XMLElement.
	virtual bool init(const DOMElement* scope=NULL);
	
	//! Implementierung der abstrakten Basisklasse Analysable.
	/*!
	 * Liefert aktuelle Messwerte fuer Latency, Delay und Queue.
	 *
	 * @see Analysable
	 */
	virtual double Measure(TypeInfo type);
	
	//! Implementierung der Basisklasse Analysable.
	/*!
	 * Liegt fuer ueber erstes Argument spezifizierte Messung ein neuer
	 * Messwert vor, so wird Messung angestossen. Es wird dabei geprueft,
	 * ob Messung zulaessig, neuer Messwert vorhanden und fuer Latenz
	 * und Verzoegung ggf. die geforerte Prioritaet uebereinstimmt.
	 *
	 * @see Analysable
	 */
	virtual void takeMeasure(const Description& descr);
 		
	/*!
	 * Implementierung der abstrakten Basisklasse Component.
	 * 
	 * @see Component
	 * @return Component::BUFFER
	 */
	virtual const int rtti() const
	{
		return Component::BUFFER;
	}

	/*! 
	 * Implementierung der abstrakten Basisklasse Component.
	 *
	 * @see Component
	 * @return "Buffer"
	 */
	virtual const std::string Name() const
	{
		return std::string ("Buffer");
	}

	//! Implementierung der abstrakten Basisklasse Component.
	/*!
	 * Die Bitmaske der erreichbaren Ziele eines Puffers ergibt sich aus
	 * der Bitmaske seines Nachfolgers in Senderichtung.
	 * 
	 * @see Component
	 */
	virtual const Bitmask& generateBitmask();

	//! Implementierung der abstrakten Basisklasse Component.
	/*!
	 * Die interne Queue und das Senderegister werden geleert.
	 * 
	 * @see Component
	 */
	virtual void reset();
	
	//! Implementierung der abstrakten Basisklasse Node.
	/*!
	 * Ein Aufruf dieser Methode sorgt dafuer, dass ein Puffer versucht, an 
	 * seine ausgehende Leitung ein Flit eines Paketes anzulegen.
	 * 
	 * @see Node
	 */
	virtual int putPacketOnLine(Wire*, bool&);
        
	//! Implementierung der abstrakten Basisklasse Node.
	/*!
	 * Ein Aufruf dieser Methode sorgt dafuer, dass ein Puffer versucht, ein 
	 * Paket von seiner eingehenden Leitung zu empfangen.
	 * 
	 * @see Node
	 */
	virtual int takePacketFromLine(Wire*, bool&);

	//! Implementierung der abstrakten Basisklasse Node.
	/*!
	 * Wird vom Vorgaenger signalisiert, dass die eingehende Leitung aktiviert
	 * wurde, ist fuer einen Puffer erst einmal nichts zu tun. Die Koroutine
	 * fordert explizit zum Empfangen auf.
	 * 
	 * @see Node
	 * @return Node::SUCCESS.
	 */
	virtual int lineActivated(Wire*, ModifierList*, 
							SignedTNodeList* =NULL, TNodeList::iterator=NLITNULL);
	
	//! Implementierung der abstrakten Basisklasse Node.
	/*!
	 * Wird einem Puffer von einem Nachfolger signalisiert, dass die 
	 * ausgehende Leitung bearbeitet wurde, so ist der Sendevorgang 
	 * abgeschlossen. 
	 * 
	 * @see Node
	 * @return Node::SUCCES.
	 */
	virtual int lineFinished(Wire*, ModifierList*, 
							SignedTNodeList* =NULL, TNodeList::iterator=NLITNULL);

	//! Signal fuer empfangenes Flit.
	/*!
	 * Diese Methode wird aufgerufen, wenn ein Flit von einer eingehenden
	 * Leitung empfangen werden konnte. Es sind bereits alle Constraints
	 * geprueft worden und das Flit muss nur noch eingepuffert werden.
	 *
	 * @see Buffer::takePacketFromLine
	 * 
	 * @param flit Das aufgenommene Flit.
	 */
	virtual void flitReceived(Flit* flit);
       
	//! Signal fuer versendetes Flit.
	/*!
	 * Diese Methode wird aufgerufen, wenn ein Flit vom Senderegister an einen 
	 * der in Frage kommenden Empfaenger versendet wurde. Es kann erst aus der
	 * Queue entfernt werden, nachdem es vollstaendig an alle Empfaenger
	 * versendet wurde und Messwerte fuer Delay und Latency bestimmt wurden. 
	 * Durch dieses Verhalten wird partieller Multicast realisiert. Unicast ist 
	 * dabei ein Spezialfall, bei dem es stets ganau einen Empfaenger gibt.
	 * 
	 * @param flit Das versendete Flit.
	 * @param complete Flag fuer vollstaendig vermitteltes Flit.
	 */
	virtual void flitTransmitted(Flit *flit, bool complete=true);

	//! Die Queue eines Puffers.
	/*! 
	 * Diese Queue dient zum Speichern der empfangenen Pakete bzw. deren Flits. Die
	 * Verteilung der verwalteten Flits erfolgt ueber ein Senderegister.
	 * 
	 * @see BufferQueue
	 */
	BufferQueue *BQ;

	// Das maeander-Flag zeigt an, dass das vorderste Flit des Puffers zu einem
	// Paket gehrt, das bezglich Shortest Path Routing auf einem Umweg ist.
	bool maeander;

	// Wird diese Signatur gesetzt, so nimmt der Puffer am Ende eines Netzzyklus
	// das maeander-Flag zurck.
	Signature solve_maeander;

protected:
		
	//! Delay innerhalb einer Queue. 
	/*!
	 * Verzoegerung eines Pakets innerhalb dieses Puffers. Der Wert wird
	 * aktualisiert, wenn der Head eines Pakets vollstaendig vermittelt
	 * wurde, und ist die Differenz aus aktuellem Takt und dem Takt des
	 * Eintreffens des Paketes (Head).
	 * 
	 * @see Buffer::flitTransmitted
	 */
	Data _delay;

	//! Latenz innerhalb einer Queue. 
	/*!
	 * Latenz eines Pakets innerhalb dieses Puffers. Der Wert wird
	 * aktualisiert, wenn der Tail eines Pakets vollstaendig vermittelt
	 * wurde, und ist die Differenz aus aktuellem Takt und dem Takt des
	 * Eintreffens des Paketes (Head).
	 * 
	 * @see Buffer::flitTransmitted
	 */
	Data _latency;
	
	/*
	// um die Verteilung bei den shared buffern hinzukriegen
	int occupiedByRouter;

	// scheduler
	unsigned int sharedNumber;
	unsigned int sharedQueueMin;
	unsigned int sharedQueueTotal;

	// die Position des vorhergehenden geshareden Buffers im Vektor
	// also wenn die 4, 5 und 6 zusammengehren, steht in 4 die 6, in
	// 5 die 4 und in 6 die 5.
	int nextShared;
*/

};

#endif
