#ifndef __CONFIGURATION_H__
#define __CONFIGURATION_H__

#include <simulation.h>
#include <utility/xmlelement.h>
#include <utility/hashmap.h>

#include <vector>

class PacketForwarding;
class Analyser;
class Component;

using std::vector;

typedef vector<Component*> ComponentVector;
typedef vector<Analyser*> AnalyserVector;


//! Netzkonfiguration einer Simulation.
/*!
 * Eine Simulation umfasst mindestens eine Netzkonfiguration, die ein
 * Verbindungsnetz und durchzufuehrende Messungen festlegt. Die aktive
 * Konfiguration kann von der Simulation angestossen werden, was einem
 * Takt des Verbindungsnetzes entspricht. In einem Takt werden werden
 * von der Koroutine die moeglichen Paketbewegungen veranlasst und
 * anschliessend von Analysern Messungen angestossen. Ist das Ende 
 * Laufzeit, festgelegt durch Takte, erreicht, so kann auf die nachfolgende 
 * Konfiguration in einer Rekonfigurationsfolge umgeschaltet werden.
 * Dabei koennen verbliebene Pakete in die neue Konfiguration uebernommen 
 * werden (experimentell).
 * 
 * @see XMLElement
 * @see Simulation
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Configuration : public XMLElement
{
// Setup.
friend class XmlSimulation;

public:

	//! Aktive Konfiguration der Simulation.
	/*!
	 * Waehrend einer Simulation ist immer genau eine Konfiguration und
	 * kann ueber diese Methode abgerufen werden.
	 * 
	 * @return Aktive Konfiguration der Simulation.
	 */
	inline static const Configuration* Instance()
	{
		return Simulation::Instance()->Active();
	}
	
	/*!
	 * Konstruktor.
	 * 
	 * @param scope Gueltigkeitsbereich in XML-Dokument.
	 */
	Configuration(const DOMElement* scope=NULL);

	//! (Virtueller) Destruktor.
	/*!
	 * Die Komponenten, die Analyser und die Koroutine werden freigegeben.
	 */
	virtual ~Configuration();

	//! Implementierung der abstrakten Klasse XMLElement.
	/*!
	 * Die Komponenten, die Analyser und die Koroutine der
	 * Konfiguration werden initialisiert.
	 *
	 * @see Component
	 * @see Analyser
	 * @see PacketForwarding
	 * @see XMLElement
	 */
	virtual bool init(const DOMElement* scope=NULL);

	//! Durchfuehrung eines Simulationstaktes. 
	/*!
	 * Der Aufruf dieser Methode entspricht einem Simulationstakt. Die
	 * Koroutine stoesst die Paketbewegung im zugehoerigen Verbingsnetz
	 * an und die Anlyser anschliessend die duchzufuehrenden Messungen.
	 *
	 * @see PacketForwarding
	 * @see Analyser
	 */
	virtual void advance();

	//! Zuruecksetzen der Konfiguration.
	/*!
	 * Die Konfiguration wird in ihren initialen Zustand zurueck versetzt, 
	 * d.h. alle Pakete entfernt und alle Schaltzustaende aufgehoben.
	 */
	virtual void reset();
	
	//! Pakete in nachfolgende Konfiguration uebertragen.
	/*!
	 * Befinden sich am Ende der Leerlaufphase noch Pakete im Verbindungsnetz
	 * dieser Konfiguration, so wird versucht, diese in die naechste
	 * Konfiguration zu uebertragen.
	 */
	virtual void transferPackets();
	
	//! Index einer Konfiguration.
	/*!
	 * Konfigurationen sind Bestandteile einer Simulation, die diese
	 * in Form einer Liste verwaltet. Der Index entspricht der Position 
	 * in dieser Liste und ist unabhaengig von der Reihenfolge der
	 * Konfigurationen in einer Rekonfigurationsfolge.
	 * 
	 * @see Simulation
	 * 
	 * @return Index der Konfiguration innerhalb der Simulation.
	 */
	inline unsigned int Index() const
	{
		return _index;
	}
	
	//! Komponenten einer Konfiguration.
	/*!
	 * Die verschiedenen Komponenten einer Konfiguration beschreiben ein
	 * Verbindungsnetz bestehend aus Quellen, Zielen, Puffern, Routern
	 * und Verbindungsleitungen.
	 * 
	 * @see Component
	 * 
	 * @param rtti Typinformation zu gesuchten Komponenten.
	 * @return Die Komponenten des Typs rtti.
	 */
	inline const ComponentVector& Components(int rtti) const
	{
		return _components.getValue(rtti);
	}

	//! Analyser einer Konfiguration.
	/*!
	 * Jede Konfiguration besitzt Analyser, die am Ende eines Taktes
	 * die durchzufuehrenden Messungen der einzelnen Komponenten anstossen.
	 * 
	 * @see Analyser
	 * 
	 * @return Die Analyser dieser Konfiguration.
	 */
	inline const AnalyserVector& Analysers() const
	{
		return _analysers;
	}

	//! Koroutine einer Konfiguration.
	/*!
	 * Jede Konfiguration besitzt eine eigene Koroutine, die fuer die
	 * Paketbewegung in einem Takt verantwortlich ist.
	 *
	 * @see PacketForwarding
	 *
	 * @return Die Koroutine dieser Konfiguration.
	 */
	inline const PacketForwarding* Coroutine() const
	{
		return _pf;
	}

	//! Betriebszeit einer Konfiguration.
	/*!
	 * Die Betriebszeit einer Konfiguration beschreibt die Anzahl der
	 * Takte, die eine Konfiguration normal betrieben wird.
	 * 
	 * @return Bertriebszeit der Konfiguration.
	 */
	inline unsigned int OperationPhase() const 
	{
		assert(_operationPhase > 0);
		
		return _operationPhase;
	}
	
	//! Leerlaufzeit einer Konfiguration.
	/*!
	 * Die Leerlaufzeit einer Konfiguration schliesst direkt an die
	 * Betriebszeit an und legt eine Anzahl von Takten fest, in der
	 * keine neuen Pakete generiert werden, die Konfiguration aber
	 * ansonsten normal betrieben wird.
	 *
	 * @return Leerlaufzeit der Konfiguration.
	 */
	inline unsigned int DrainPhase() const 
	{
		return _drainPhase;
	}

	//! Umschaltzeit einer Konfiguration.
	/*!
	 * Die Umschaltzeit einer Konfiguration schliesst direkt an die 
	 * Leerlaufzeit an und legt eine Anzahl von Takten fest, die fuer
	 * das Umschalten auf die nachfolgende Konfiguration angenommen
	 * werden soll. In dieser Zeit erfolgen keine Paketbewegungen und
	 * keine Messungen.
	 *
	 * @return Umschaltzeit der Konfiguration.
	 */
	inline unsigned int ReconfigurationPhase() const 
	{
		return _reconfigPhase;
	}

	//! Typ einer Konfiguration.
	/*!
	 * Der Typ ist entweder dynamisch oder statisch. Je nach Typ
	 * verhlt sich die Vorbereitungsphase (DrainPhase) verschiedenartig.
	 * 
	 * @return Typ der Konfiguration.
	 */

	inline bool Dynamic() const
	{
		return _dynamic;
	}

	//! Nachfolgende Konfiguration in Rekonfigurationsfolge.
	/*!
	 * In einer Rekonfigurationsfolge kann eine Konfiguration genau eine
	 * Nachfolgekonfiguration besitzen, auf die am Ende ihrer Laufzeit
	 * umgeschaltet wird. 
	 * Existiert keine Nachfolgekonfiguration, so ist
	 * die Laufzeit (theoretisch) unendlich.
	 * 
	 * @return Nachfolgende Konfiguration oder NULL, falls keine existiert.
	 */
	inline const Configuration* Next() const
	{
		return _nextConfig;
	}
	
	//! Zugriff auf DOM-Element mit Parametern einer Konfiguration.
	/*!
	 * Der Gueltigkeitsberich einer Konfiguration im DOM-Dokument einer
	 * Simulationsbeschreibung beginnt mit dem Top-Level Element, welches
	 * die Netzkonfiguration enthaelt. Die Parameter, welche Routing- und
	 * Switchingverfahren, sowie die Koroutine festlegen, befinden sich
	 * in einem speziellen Element, welches entweder dem globalen oder dem 
	 * lokalen Menue (falls existent) entstammt. Diese Methode liefert
	 * das fuer diese Konfiguration relevante Element.
	 * 
	 * @return DOM-Element mit Parametern einer Konfiguration.
	 */
	inline const DOMElement* Settings() const
	{
		return _settings; 
	}
	
	//! Tabellen, die den Zusammenhang zur nchsten Konfiguration definieren
	vector<int> _nextBuffer;        // fr Buffer
	vector<int> _nextTarget;        // fr Targets

private:
	
	typedef IntMap< ComponentVector > RTTIMap;

	//! Index der Konfiguration.
	unsigned int _index;
	
	//! Koroutine.
	PacketForwarding *_pf;
		
	//! Analyser der Konfiguration.
	vector<Analyser*> _analysers;

	//! Komponenten der Konfiguration nach RTTI.
	RTTIMap _components;

	//! Element mit Parametern fuer diese Konfiguration.
	const DOMElement* _settings;

	//! Bertriebszeit.
	unsigned int _operationPhase;
	//! Leerlaufzeit.
	unsigned int _drainPhase;
	//! Umschaltzeit.
	unsigned int _reconfigPhase;

	//! Flag fr dynamische Rekonfiguration.
	bool _dynamic;

	//! Nachfolgende Konfiguration.
	Configuration* _nextConfig;
};


inline const PacketForwarding* Coroutine()
{
	return Configuration::Instance()->Coroutine();
}

inline const ComponentVector& Components(int rtti)
{
	return Configuration::Instance()->Components(rtti);
}

#endif
