#ifndef __DLHANDLER_H__
#define __DLHANDLER_H__

#include <configuration.h>
#include <buffer.h>

//! DLHandler hilft Deadlocks lsen, die durch Umwegpakete entstehen
/*!
 * Diese Klasse implementiert ein Analysationstool zur Behebung von Dead-
 * locks, die infolge einer Umadressierung von Paketen entstanden. Folgende
 * Dinge sind damit machbar: 
 * 
 * - Tabellieren der Sende-Abhngigkeiten der Buffer in einem Takt.
 * - Aufstellen von Switching-Trees zur Erkennung von Buffern, die unter
 *   einander einen Deadlock lsen sollen.
 * - Auffinden aller Puffer, die eine "Unendlich"-Markierung erhalten. Dabei
 *   sind "unendlich" grosse Puffer nicht fhig, von endlich grossen ein
 *   Flit zu empfangen.
 *    
 * @author Christian Zimmermann <zimmermc@ee.ethz.ch>
 */

// Bume lassen sich in 6 Typen klassifizieren:
// E  1 EMPTY = Wurzel existiert nicht.
// F  2 FREE = Wurzel hat keine Nachfolger.
// N  3 NORMAL = Baum enthlt keine loops.
// S  4 SOLVABLE = Baum enthlt nur loops, die aus Wurzel und Blttern bestehen.
// U  5 UNSOLVABLE = Baum enthlt loops mit inneren nodes.
// D  6 DIVISIBLE = Baum enthlt Teile, die lsbar sind.
// ?  0 UNKNOWN
enum e_type {UNKNOWN, EMPTY, FREE, NORMAL, SOLVABLE, UNSOLVABLE, DIVISIBLE};

// U und D sind "intricate", die anderen nicht.
// S und D sind "passable", die anderen nicht.

class TreeNode;

typedef struct downProtocol1
{
	e_type downType;
	vector<TreeNode*> leaves;
} downProtocol;

typedef struct pathO1
{
	unsigned int tree,path;
	vector<unsigned int> pSetO;
} pathO; // "path as ordered list"


/*! TreeNode ist ein Knoten in einem Switching-Baum
 * 
 * Er hat gewisse Funktionalitt, indem er weiss, welcher Pfad von der
 * Wurzel zu ihm fhrt.
 */

class TreeNode
{
public:	
	// Standard-Konstruktor
	TreeNode();

	// Destruktor: dealloziert alle Nachfolger
	~TreeNode()
	{
		for(vector<TreeNode*>::iterator tnit=next.begin(); tnit!=next.end(); tnit++)
			delete (*tnit);
	}

	// Alternativ-Konstruktor: mit Nummer-Vorgabe und Vorgngerknoten
	TreeNode(int givenNr, TreeNode* wherefrom);

	// Jeder TreeNode weiss, ob er ein Blatt ist.
	inline bool isLeaf() {return (next.size()==0);}

	// Jeder Tree weiss, ob er eine Astgabel ist.
	inline bool isCrotch() {return (next.size()>1);}

	// passedFor gibt an, wann testNr zum letzten mal passiert wurde. 0 wenn nie.
	// Diese Funktion wird zum Aufbauen eines tree bentigt.
	unsigned int passedFor(unsigned int testNr);

	// downAnalyze() sollte nur fr roots angewendet werden. Allerdings ruft nur
	// SwitchingTree diese Methode auf, auch nur fr roots. 
	//
	// Funktionsweise: Jeder node fragt alle seine Nachfolger, was ihre downAnalyze()
	// ist. Bltter hingegen wissen: sie sind FREE und haben leafNr[0]=nr. So wird 
	// rekursiv der ganze Baum analysiert. 
	downProtocol downAnalyze();

	std::string toString();

	// Variablen sind alle public, weil diese Klasse eher wie Structs behandelt
	// wird als wie Objekte.
	
	// Die Nummer des dazugehrigen Puffers
	unsigned int nr;

	// Anzahl von Nachfolgern 0 bis beliebig viele
	vector<TreeNode*> next;

	// Der Vorgnger, kann auch NULL sein.
	TreeNode* last;

	// Der Pfad, absteigend sortiert natrlich
	vector<unsigned int> path;	
};

/*! SwitchingTree verwaltet einen bestimmten Baum
 *
 */
class SwitchingTree
{
public:
	// Alternativ-Konstruktor mit Angabe des Root, zu Beginn nicht identifiziert.
	SwitchingTree(TreeNode* tn) {root = tn; identified = false; }

	// Destruktor, lsst root verschwinden, demnach auch alle Nachfolger davon.
	~SwitchingTree() {delete root;}

	TreeNode* getRoot() {return root;}

	e_type getType() 
	{
		if(identified) 
			return treeType; 
		return UNKNOWN;
	}

	vector< vector<unsigned int> > getPaths() {return paths;}

	std::string strGetType();

	std::string toStringPaths();
	
	// Charakterisiert den Typ von Baum, der in root ist. Wird nur durchge-
	// fhrt, wenn nicht identified. Die Methode arbeitet mit downAnalyze().
	void identifyType();
	
private:
	// Markierung fr identifizierten Baum
	bool identified;
	
	// Typ des Baums
	e_type treeType;

	// Pointer auf Wurzel des Baums
	TreeNode* root;	

	// Die eruierten Pfade von SOLVABLE-Bumen
	vector< vector<unsigned int> > paths; 
};

/*! DLHandler ist ein Objekt, das Deadlocks in Netzkonfigurationen behandelt
 *
 * Es kann dabei verschiedene Switching-Bume aufstellen und analysieren.
 */
class DLHandler
{

public:
	
	void setConfig(Configuration* conf)
	{
		_aimedConfig = conf;
		// jede Konfig ist zu Beginn als kritisch einzustufen.
		critical = true;
	}
	
	DLHandler();

	//! Ist die Netzsituation (noch) kritisch?
	/*
	 * Mit optimaler Laufzeit entscheidet diese Funktion, ob Umkehrpakete
	 * im Netz sind.
	 */
	bool Critical();

	//! Stelle die bentigten Tabellen auf
	/*
	 * Diese Methode fllt die Liste _bufferOfIndex und die _graphTable.
	 */
	void setTables();

	// Schreibe die Tabellen hin
	std::string toStringGraphTable();

	//! Switching Trees aufstellen und guten Buffer auswhlen
	/*
	 * Ein Buffer wird nur dann ausgewhlt, wenn ein Deadlock anhand eines
	 * Switching Tree entdeckt wurde und dieser auch lsbar ist mit dem 
	 * Forcieren des Buffers. Wird kein Deadlock entdeckt oder ist keine
	 * Lsung des Deadlock vorhanden, so wird _chosenBuffer null sein.
	 */
	void setTrees();

	// Zeichne die Bume auf
	std::string toStringTrees(bool with_paths);

	int maxType()
	{
		int ret_ui = 0;
		for(unsigned int k=0;k<_trees.size();k++)
			if(_trees[k]->getType()>ret_ui)
				ret_ui=_trees[k]->getType();
		return ret_ui;
	}

	//! Forciere einen mglichen Buffer und seine Nachfolger.
	/*
	 * Der Name dieser Methode ist etwas missverstndlich. Es wird nun
	 * kein manuelles Switching durchgefhrt, sondern es werden nur Puffer
	 * als "unendlich" gross signiert, die von der normalen Coroutine ge-
	 * steuert werden, aber nicht Flits von normalen Puffern akzeptieren.
	 */
	bool forceSwitching();

	bool Solution() {return !_allunsolvable;}

private:

	//! Addiere rekursiv alle fehlenden Knoten eines Switching Tree
	/*
	 * Startet man diese Methode mit einem Root-Node, so hat man gleich
	 * den ganzen Switching Tree produziert.
	 */
	void addNodes(TreeNode* where);

	Configuration* _aimedConfig;

	bool critical;

	bool _allunsolvable;

	vector< vector<unsigned int> > _graphTable;

	vector<Buffer*> _bufferOfIndex;

	vector<SwitchingTree*> _trees;
};

#endif
