#ifndef __CINPARSER_H__
#define __CINPARSER_H__

#include <utility/hashmap.h>

#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/framework/psvi/PSVIHandler.hpp>
#include <xercesc/framework/psvi/PSVIElement.hpp>
#include <xercesc/framework/psvi/XSElementDeclaration.hpp>

XERCES_CPP_NAMESPACE_USE

typedef StringMap<XSElementDeclaration*> SchemaInformation;

//! XML-Parser fuer Simultionsbeschreibungen.
/*! 
 * Diese Klasse implementiert einen XML-Parser fuer CINSim, der die 
 * Simulationsbeschreibungen parst und als DOM-Baum zur Verfuegung
 * stellt.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class CINParser : public XercesDOMParser
{
public:

	//! Handler fuer PSVI-Events.
	/*!
	 * Diese Klasse faengt PSVI-Events des Parsers ab und erstellt damit
	 * eine Hashmap mit Schema-Informationen zu allen geparsten Elementen.
	 */
	class PSVIHandlerImpl : public PSVIHandler
	{
	public:
	
		//! Konstruktor.
		PSVIHandlerImpl(SchemaInformation &map) : _map(map) {} 
	
		//! Implementierung der abstrakten Klasse PSVIHandler.
		virtual void handleElementPSVI(
				const XMLCh* const localName, 
				const XMLCh* const uri, 
				PSVIElement *elementInfo);

		//! Implementierung der abstrakten Klasse PSVIHandler.
		virtual void handleAttributesPSVI(const XMLCh* const name, 
				const XMLCh* const uri, 
				PSVIAttributeList *psviAttributes) { };    

	private:

		//! Map mit Schema-Informationen geparster Elemente.
		SchemaInformation& _map;
	};
	
	//! Enumeration zur Klassifizierung von Parse-Fehlern.
	/*!
	 *
	 * ErrType_Warning 	- Warnung<br>
	 * ErrType_Error	- Fehler beim Erstellen des Dom-Trees (Validierung)<br>
	 * ErrType_Fatal	- Fehler beim Lesen des XML-Files (Syntax)<br>
	 * ErrType_Check	- Benutzerdefinierter Fehler<br>	 
	 * ErrType_Unkown	- unspezifizierter Fehler
	 */
	enum ErrTypes
	{
		ErrType_Warning,
		ErrType_Error,
		ErrType_Fatal,
		ErrType_Check,
		ErrTypes_Unknown
	};

	//! Konstruktor.
	/*!
	 * Erstellt ein neues Objekt der Klasse CINParser anhand der Informationen
	 * aus der Systemkonfiguration und initialisiert Xerces-C. Die Fehlerbehandlung 
	 * erfolgt durch eine Instanz der Klasse ErrHandler. 
	 *
	 * @see ConfigDirectives
	 * @see ErrHandler
	 */ 
	CINParser();
     
	//! Simulationsbeschreibung parsen.
	/*!
	 * Diese Methode startet das Parsen einer Simulationsbeschreibung und 
	 * verwendet einen Pointer auf die je nach Systemkonfiguration zu 
	 * verwendende Parse-Methode. Es stehen auch alle Parse-Methoden der 
	 * Klassen XercesDOMParser zur Verfuegung.
	 *
	 * @return true, false parsen erfolgreich.
	 */
	inline bool parseFile(const char* xmlFile)
	{
		return (this->*parseMethod)(xmlFile);
	}
  
	//! Re-Implementierung der Klasse XercesDOMParser.
	/*!
	 * Fuer das progressive parsen steht ein weiterer Fehlertyp zu Verf�gung.
	 *
	 * @see parseProgressive
	 * @see ErrTypes
	 */
	virtual void error
	(
		const   unsigned int                errCode
		, const XMLCh* const                msgDomain
		, const ErrTypes                    errType
		, const XMLCh* const                errorText
		, const XMLCh* const                systemId
		, const XMLCh* const                publicId
		, const XMLSSize_t                  lineNum
		, const XMLSSize_t                  colNum
	);

	//! Anzahl der aufgetretenen Fehler.
	/*!
	 * Diese Methode liefert die Anzahl der von der verwendeten Instanz 
	 * der Klasse ErrHandler registrierten Fehler.
	 * 
	 * @see ErrHandler
	 *
	 * @return Anzahl der aufgetretenen Parse-Fehler.
	 */
	int getParseErrors() const;

	//! Zuruecksetzen des Fehler-Zaehlers.
	/*!
	 * Diese Methode setzt den Zaehler der verwendeten Instanz der Klasse 
	 * ErrHandler zurueck.
	 *
	 * @see ErrHandler
	 */
	void resetErrors();
	
	/*! Schema-Informationen zu geparseten Elementen.
	 *
	 * @see PSVIHandlerImpl
	 * @returns Schema-Informationen zu geparsten Elementen.
	 */
	inline const SchemaInformation* getSchemaInformation() const
	{
		return &_elements;
	}

	
protected:
  
	//! Progressive Parse-Methode fuer Simulationsbeschreibungen. 
	/*!
	 * Diese Methode bietet die Moeglichkeit ein XML-File progressive zu
	 * parsen und dabei neben der Schema-Validierung, eigene Constraints
	 * zu parsen.
	 *
	 * @param xmlFile XML-Datei mit Simulationsbeschreibung.
	 * @return true, falls parsen erfolgreich.
	 */	
	bool parseProgressive(const char* xmlFile);

	//! Standard Parse-Methode fuer Simulationsbeschreibungen.
	/*
	 * Diese Methode benutzt die Methode void parse(char*) der Klasse 
	 * XercesDOMParser, um Simulationsbeschreibungen mit oder ohne 
	 * Schema-Validierung zu parsieren.
	 *
	 * @param xmlFile XML-Datei mit Simulationsbeschreibung.
	 * @return true, falls parsen erfolgreich.
	 */
	bool parse(const char* xmlFile);

	//! Pointer auf Parse-Methode.
	bool (CINParser:: *parseMethod)(const char* xmlFile);

	
private:

	//! HashMap <Name,Info> mit Schema-Informationen geparster Elemente.
	SchemaInformation _elements;
};

#endif
