#ifndef __CMDPARSER_H__
#define __CMDPARSER_H__

#include <utility/configdirectives.h>

#include <config.h>
#include <string>
#include <list>
#include <stdio.h>
#include <string.h>

#ifdef HAVE_GETOPT_H
#define HAVE_GETOPT_LONG
#include <getopt.h>
#endif

using std::string;
using std::list;

//! Parser fuer Kommandozeilenoptionen.
/*!
 * Diese (abstrakte) Klasse realisiert die Infrastruktur zum Parsen
 * der Kommandozeilenoptionen aller Programme des CINSim-Kerns. Die
 * zulaessigen Optionen, die fuer jedes Programm ueber die Argumentliste
 * angegeben werden koennen, werden ueber die abstrakten Methoden Flags()
 * und LongOpts() spezifiziert. Die geparsten Optionen werden als 
 * Konfiguration verwaltet und koennen auf die globale Konfiguration
 * einer Simulation uebertragen werden.
 * 
 * @see ConfigDirectives
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class CmdParser : public ConfigDirectives
{
public:

	//! Parsen der Kommandozeilenoptionen.
	/*!
	 * Die Argumentliste wird untersucht und die zulaessigen Optionen,
	 * sowie die zu verwendende Simulationsbeschreibung in die lokale
	 * Konfiguration uebertragen.
	 * 
	 * @param argc Anzahl der zu untersuchenden Argumente.
	 * @param argv Liste der Argumente.
	 */
	virtual bool parseArgs(int argc, char** argv);
	
	//! Ausgabe von Hinweisen zur Benutzung.
	/*!
	 * Es werden Informationen zu den verfuegbaren Kommandozeilenoptionen
	 * ausgegeben.
	 * 
	 * @param argc Anzahl der zu untersuchenden Argumente.
	 * @param argv Liste der Argumente. 
	 */
	virtual void printUsage(int argc, char** argv) const;
	
protected:
	
	//! Kurzbeschreibung des Programms.
	/*!
	 * Eine kurze Information zum Programm fuer das die Kommandozeilenoptionen
	 * analysiert werden. Wird von printUsage(...) verwendet.
	 *
	 * @return Kurzer Informationsstring.
	 */
	virtual const char* Info() const=0;
	
	//! Verfuegbare Optionen (Kurzform)
	/*!
	 * Liefert einen String mit allen verfuegbaren Optionen. Jede Option 
	 * besteht aus einen einzelnen Zeichen, das der Kommandozeile mit einen 
	 * vorangestellten '-' uebergeben werden kann. Folgt dem Zeichen im String
	 * ein ':', so wird zusaetzlich ein Argument erwartet.
	 * 
	 * @see getopt
	 * 
	 * @return Zulaessige Kommandozeilenoptionen.
	 */
	virtual const char* Flags() const=0;
	 
#ifdef HAVE_GETOPT_LONG
	//! Verfuegbare Optionen (Langform)
	/*!
	 * Falls vom Compiler unters�tzt, liefert diese Methode eine Struktur,
	 * die zulaessige Kommandozeilenoptionen in Kurz- und Langform definiert,
	 * sowie eventuell erwartete Argumente spezifiert. 
	 * 
	 * @see getopt_long
	 * 
	 * @return Zulaessige Kommandozeilenoptionen.
	 */
	virtual const struct option* LongOpts() const=0;
#endif
	
	//! Auswertung von Wertzuweisungen an Variablen.
	/*!
	 * Analysiert Strings mit Wertzuweisungen an Variablen der "a=1;b=2" und
	 * legt diese in der lokalen Konfiguration ab.
	 * 
	 * @param arg String Wertzuweisungen an Variablen.
	 */
	bool parseVariables(string arg);
	
	//! Hilfsmethode zur Zerlegung von Strings.
	/*!
	 * Zerlegt eine String bezueglich uebergebener Trennzeichen und legt
	 * Fragmente in drittem Argument ab.
	 * 
	 * @param text Der zu zerlegende String.
	 * @param separators Die Trennzeichen.
	 * @param words Liste fuer Stringfragmente.
	 */
	void split(const string& text, const string& separators, list<string>& words) const;
};


//! Kommandozeilenparser fuer Simulationskern.
/*!
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class CINSimCmdParser : public CmdParser
{
public:

	/*!
	 * Implementierung der abstrakten Klasse CmdParser.
	 * 
	 * @see CmdParser::Info
	 */
	const char* Info() const
	{
		static const char* info = "Simulation engine";
		return info;
	}
	
	/*!
	 * Implementierung der abstrakten Klasse CmdParser.
	 * 
	 * @see CmdParser::Flags
	 */
	const char* Flags() const
	{
		static const char* flags = "cd:ef:hi:lmn:p:s:tvC:G:P:R:S:";
		return flags;
	}

#ifdef HAVE_GETOPT_LONG	
	/*!
	 * Implementierung der abstrakten Klasse CmdParser.
	 * 
	 * @see CmdParser::LongOpts
	 */
	const struct option* LongOpts() const
	{
		static struct option longopts[] = 
		{
			{"help", 0, 0, 'h'},
			{"verbose", 0, 0, 'v'},
			{"timer", 0, 0, 't'},
			//{"quiet", 0, 0, 'q'},
			{"extended-validation", 0, 0, 'e'},
			{"schema-validation", 0, 0, 'c'},
			{"minimal-validation", 0, 0, 'm'},
			{"plugin-list", 0, 0, 'l'},
			{"config-file", 1, 0, 'f'},
			{"schema", 1, 0, 's'},
			{"namespace", 1, 0, 'n'},
			//{"variables", 1, 0, 'V'},
			{"nice-level", 1, 0, 'i'},
			{"random-generator", 1, 0, 'G'},
			{"random-stream", 1, 0, 'S'},
			{"data-file", 1, 0, 'd'},
			{"plot-file", 1, 0, 'p'},
			{"confidence", 1, 0, 'C'},
			{"precision", 1, 0, 'P'},			
			{"repetitions", 1, 0, 'R'},	
			{0, 0, 0, 0}
		};

		return longopts;
	}
#endif
};


/*!
 * Kommandozeilenparser fuer CINSim XML-Parser.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 * @see CmdParser
 */
class CINParserCmdParser : public CmdParser
{
public:

	/*!
	 * Implementierung der abstrakten Klasse CmdParser.
	 * 
	 * @see CmdParser::Info
	 */
	const char* Info() const
	{
		static const char* info = "XML parser";
		return info;
	}

	/*!
	 * Implementierung der abstrakten Klasse CmdParser.
	 * 
	 * @see CmdParser::Flags
	 */
	const char* Flags() const
	{
		static const char* flags = "cef:hmn:s:tv";
		return flags;
	}

#ifdef HAVE_GETOPT_LONG	
	/*!
	 * Implementierung der abstrakten Klasse CmdParser.
	 * 
	 * @see CmdParser::LongOpts
	 */
	const struct option* LongOpts() const
	{
		static struct option longopts[] = 
		{
			{"help", 0, 0, 'h'},
			{"verbose", 0, 0, 'v'},
			{"extended-validation", 0, 0, 'e'},
			{"schema-validation", 0, 0, 'c'},
			{"minimal-validation", 0, 0, 'm'},
			{"config-file", 1, 0, 'f'},
			{"schema", 1, 0, 's'},
			{"namespace", 1, 0, 'n'},
			{"timer", 0, 0, 't'},
			{0, 0, 0, 0}
		};

		return longopts;
	}
#endif
};

#endif
