#ifndef __ESTIMATION_H__
#define __ESTIMATION_H__

#include <cfloat>

//! Verwaltung und Auswertung von Messwerten.
/*!
 * Diese Klasse ist der Ausgangspunkt fuer alle Arten von Abschaetzungen.
 * Es koennen Messwerte uebergeben und an Kontrollpunkten die aktuellen
 * Abschaetzungen abgefragt werden. Es wird angenommen, dass alle Werte einem 
 * stationaeren Prozess entstammen und die Werte einer eventuell vorhandenen 
 * initialen transienten Phase nicht uebergeben werden.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Estimation
{
public:

	//! Verfuegbare Arten der Auswertung.
	enum Type{ MEAN, QUANTILE, OTHER };
	
	//! Konstruktor.
	/*!
	 * Die Auswertung wird bezueglich der uebergebenen Konfidenz initialisiert.
	 * 
	 * @param confidence Das Konfidenzniveau fuer die Auswertung.
	 */
	Estimation(double confidence) : _confidence(confidence) 
	{	
		reset(); 
	}
	
	//! (Virtueller) Destruktor.
	virtual ~Estimation() {};
		
	//! Art der Auswertung.
	/*!
	 * Es koennen derzeit die Paramter Mittwelwert und p-Quantile untersucht 
	 * werden.
	 *
	 * @see Estimation::Type
	 * @return Identifier fuer Art der Auswertung.
	 */
	virtual int estimationType() const = 0;
	
	//! Ein neues Datum aufnehmen.
	/*!
	 * Diese Methode uebergibt eine neuen Messwert an die Auswertung. Wird
	 * ein Kontrollpunkt erreicht, so werden die Werte der Abschaetzung 
	 * aktualisiert.
	 *
	 * @param value Ein neues Datum fuer Abschaetzung eines Parameters.
	 * @return true, falls Kontrollpunkt erreicht.
	 */
	virtual bool advance(double value) = 0;
	
	//! Test auf erreichten Kontrollpunkt.
	/*!
	 * In sequentiellen Simulationen erfolgt die Auswertung der Simulationsdaten
	 * waehrend der Simulation. Um den Aufwand fuer die Berechnung zu minimieren
	 * werden die Abschaetzungen aber nur an Kontrollpunkten aktualsiert. Diese
	 * Methode zeigt an, ob ein Kontrollpunkt erreicht wurde. Ist dies der Fall,
	 * so sollten Abschaetzung, Varianz, Fehler und Freiheitsgrade aktualsiert
	 * worden sein.
	 *
	 * @see add
	 * @see estimate
	 * @see variance
	 * @see freedom
	 * @see error
	 * 
	 * @return true, falls Kontrollpunkt erreicht.
	 */
	virtual bool checkPoint() const = 0;
	
	//! Zuruecksetzen der Abschaetzung.
	/*!
	 * Die ermittelten Werte zur Abschaetzung des untersuchten Parameters werden 
	 * zurueckgesetzt. In abgeleiteten Klassen sollte zusaetzlich die Verwaltung 
	 * der Simulationsdaten zurueckgesetzt werden.
	 */
	virtual void reset()
	{
		_estimate = 0.0;
		_variance = 0.0;
		_freedom = 0.0;
		_error = DBL_MAX;
		
		_transient = 0;
	}
	
	//! Laenge der intialen transienten Phase.
	/*!
	 * Die Laenge der initialen transienten Phase kann fuer Heuristiken zur
	 * Bestimmung der Abstaende zwischen einzelnen Kontrollpunkten verwendet
	 * werden und wird ueber diese Methode gesetzt.
	 *
	 * @param transient Laenge der initialen transient Phase. 
	 */
	inline void setTransient(unsigned int transient)
	{
		_transient = transient;
	}
	
	//! Abschaetzung des untersuchten Parameters.
	/*!
	 * Diese Methode liefert die aktuelle Abschaetzung des untersuchten 
	 * Parameters.
	 *
	 * @note Aktualsierung erfolgt normalerweise nur an Kontrollpunkten.
	 * @return Aktueller Wert fuer abzuschaetzenden Parameter.
	 */
	inline double estimate() const
	{
		return _estimate;
	}
	
	//! Varianz der Abschaetzung.
	/*!
	 * Diese Methode liefert die aktuelle Varianz der Abschaetzung des
	 * untersuchten Parameters.
	 *
	 * @note Aktualsierung erfolgt normalerweise nur an Kontrollpunkten.
	 * @return Varianz der Abschaetzung.
	 */
	inline double variance() const
	{
		return _variance;
	}

	//! Freiheitsgrade fuer Konfidenzintervall.
	/*!
	 * Zur Konstruktion eines Konfidenzintervalls anhand der aktuellen
	 * Abschaetzung und Varianz werden je nach Verfahren unterschiedliche
	 * Freiheitsgrade fuer die t-Verteilung angenommen. Diese Methode liefert
	 * den aktuellen Wert.
	 * 
	 * @note Aktualsierung erfolgt normalerweise nur an Kontrollpunkten.
	 * @return Freiheitsgrade fuer Konfidenzintervall.
	 */
	inline double freedom() const
	{
		return _freedom;
	}
	
	//! Fehler der Abschaetzung.
	/*!
	 * Diese Methode liefert den aktuellen Fehler der Abschaetzung auf
	 * Basis der uebegebenen Daten. Der Fehler ergibt sich aus der 
	 * relativen Haelfte eines konstruierten Konfidenzintervalls.
	 *
	 * @note Aktualsierung erfolgt normalerweise nur an Kontrollpunkten.
	 * @return Fehler der Abschaetzung.
	 */
	inline double error() const
	{
		return _error;
	}
	
	//! Aktualisierung einer Ergebnisdatenstruktur.
	/*!
	 * An Kontrollpunkten werden die Werte fuer Abschaetzung, Varianz,
	 * Freiheitsgrade und Fehler aktualsiert. Diese Werte koennen dann
	 * entweder einzeln abgerufen oder ueber diesen Operator in eine
	 * Ergebnisdatenstruktur uebertragen werden. Ein Zaehler fuer die
	 * erreichten Kontrollpunkte wird dabei erhoeht.
	 * 
	 * @see checkPoint
	 * @param result Zu aktualisierende Ergebnisdatenstruktur.
	 */
	template<typename C>
	inline const Estimation& operator>>(C& result) const
	{
		result.estimate = _estimate;
		result.variance = _variance;
		result.freedom = _freedom;
		result.error = _error;

		result.chkpnts++;
	}
	
protected:

	//! Aktueller Wert der Abschaetzung.
	double _estimate;

	//! Aktuelle Varianz der Abschaetzung.
	double _variance;

	double _freedom;
	
	//! Aktueller Fehler.
	double _error;

	//! Konfidenzlevel.
	double _confidence;

	//! Laenge der initialen transienten Phase.
	unsigned int _transient;
};

#endif
