#ifndef __INDEPENDENTMEAN_H__
#define __INDEPENDENTMEAN_H__

#include <estimation.h>

//! Bestimmung von Mittelwerten im unkorrelierten Fall.
/*!
 * Diese Klasse realisiert die Abschaetzung von Mittelwerten aus
 * Simulationsdaten unter der Annahme, dass die Daten unabhaengig 
 * und identisch verteilt sind. Es wird daher die klassische Konstruktion
 * eines Konfidenzintervalls verwendet. Die Verwaltung der Daten 
 * X_1,X_2,.. erfolgt in Form der Summen SUM(X) und SUM(X^2) aus denen
 * Mittelwert und Varianz nach n Werten bestimmt werden koennen und somit
 * nicht alle Simulationsdaten gespeichert werden muessen.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class IndependentMean : public Estimation
{
public:

	//! Konstruktor.
	/*!
	 * Die Abschaetzung wird initialisiert.
	 *
	 * @param confidence Das Konfidenzniveau der Abschaetzung.
	 */
	IndependentMean(double confidence);

	//! Implementierung der Klasse Estimation.
	/*!
	 * Diese Ableitung der Klasse Estimation bestimmt Mittelwerte. 
	 * 
	 * @see Estimation
	 * @return Estimation::MEAN
	 */
	int estimationType() const
	{
		return Estimation::MEAN;
	}
	
	//! Implementierung der Klasse Estimation.
	/*!
	 * Es werden die Summen SUM(X) und SUM(X^2) der Simulationsdateb
	 * X_1,X_2,.. aktualisiert und an Kontrollpunkten eine aktuelle
	 * Abschaetzung des Mittelwertes bestimmt.
	 * 
	 * @see Estimation
	 */
	bool advance(double value);

	//! Implementierung der Klasse Estimation.
	/*!
	 * Kontrollpunkte werden bei diesem Verfahren in konstanten Abstaenden 
	 * gesetzt.
	 *
	 * @see Estimation
	 */
	bool checkPoint() const;

	//! Implementierung der Klasse Estimation.
	/*!
	 * Die aktuelle Abschaetzung und die Verwaltungsinformationen werden
	 * zurueckgesetzt.
	 * 
	 * @see Estimation
	 */
	void reset();
	
protected:
	
	//! Sample-Groesse.
	double _ssize;

	//! Summe der Quadrate.
	double _ssquares;

	//! Summe.
	double _ssum;

	//! Z(alpha)
	double _zdist;
};

#endif
