#ifndef __SPECTRALQUANTILE_H__
#define __SPECTRALQUANTILE_H__

#include <estimation.h>

class PSquareQuantile;


//! Quantilenabschaetzung durch Spektral-Methode.
/*!
 * Das Verfahren basiert auf Raatikainen93, es wird aber immer nur
 * ein Quantil bestimmt. Die Abschaetzung des Quantils erfolgt durch
 * Anwendung des PP-Algorithmus, die Abschaetzung der Varianz durch
 * Spektralanalyse. Der Verwaltung der Daten erfolgt in Form eines
 * Datenpuffers (Segment) konstanter Groesse. Wurden die Daten eines
 * Segmentes gesammelt, so werden die Spektraldichten bestimmt und
 * die Summen der Spekraldichten ueber alle Segmente aktualsiert.
 * Diese werden dann fuer die Abschaetzung der Varianz verwendet.
 * 
 * @see PSquareQuantile
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de> 
 */
class SpectralQuantile : public Estimation
{
public:

	//! Konstruktor.
	/*!
	 * Die Abschaetzung wird initialisiert.
	 *
	 * @param confidence Das Konfidenzniveau der Abschaetzung.
	 * @param p Paramter fuer p-Quantil (0<p<1).
	 */	
	SpectralQuantile(double confidence, double p=0.5);

	//! Destruktor.
	~SpectralQuantile();
	
	//! Implementierung der Klasse Estimation.
	/*!
	 * Diese Ableitung der Klasse Estimation bestimmt Quantile.
	 * 
	 * @return Estimation::QUANTILE
	 */
	int estimationType() const
	{
		return Estimation::QUANTILE;
	}
	
	//! Implementierung der Klasse Estimation.
	/*!
	 * Anhand des PP-Algorithmus wird Abschaetzung des Quantils aktualisiert,
	 * ansonsten Daten fuer ein Segment gesammelt. An einem Kontrollpunkt
	 * wird der binaere Prozess I(x_p) anhand der gesammelten Daten bestimmt,
	 * die Spektraldichten und daraus die Varianz der Abschaetzung berechnet. 
	 * Der Datenpuffer wird dann mit den Daten fuer den naechsten Kontrollpunkt 
	 * ueberschrieben.
	 * 
	 * @see Estimation.
	 */
	bool advance(double value);

	//! Implementierung der Klasse Estimation.
	/*!
	 * Ein Kontrollpunkt wird nach Aufnahme der Daten fuer ein Segment
	 * gesetzt.
	 * 
	 * @see Estimation
	 */
	bool checkPoint() const;
	
	//! Implementierung der Klasse Estimation.
	/*!
	 * Der PP-Algorithmus, die aktuellen Abschaetzungen und der Datenspeicher
	 * werden zurueckgesetzt.
	 * 
	 * @see Estimation
	 */
	void reset();
	
	
protected:
	
	//! 1 - Konfidenzniveau.
	double _alpha;
	
	//! Aktuell bearbeitetes Segment.
	unsigned int _s;
	
	//! Puffer fuer Werte eines Segmentes.
	double* _buffer;
	
	//!	Groesse des Puffers.
	unsigned int _bsize;

	//! Anzahl gesammelter Werte fuer aktuelles Segment.
	unsigned int _bcount;

	//! Frequenzen.
	double* _fj;
	
	//! Summe fuer gemittelte Periodogramme ueber s Segemente.
	double* _h_fj;
	
	//! Zeilensumme (c1j)^i, j=1..K, i=1..4, der Matrix C (least-squares).
	double* _csum;
	
	//! Approximation des p-Quantils.
	PSquareQuantile* _psq;
};

#endif
