#ifndef __FLIT_H__
#define __FLIT_H__

#include <cstdlib>
#include <string>
#include <vector>

class Flit;
class Packet;
class Buffer;

typedef std::vector<Flit*> Frame;
typedef std::vector<Flit*>::iterator FlitIterator;

struct t_storage_info
{
	Buffer* where;	// Aktueller Puffer-Knoten
	signed int since;	// Takt der Einpufferung
	
	//! Vergleich zweier Positionen.
	bool operator==(const t_storage_info& si) const { return where == si.where; }
};

//! Paketfragmente.
/*!
 * Ein Paket zur Nachrichtuebertragung besteht aus mindestens einem
 * Fragment, genannt Flit. Diese Klasse beschreibt ein solches Flit.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Flit
{
friend class TargetBuffer;			// Pakete loeschen
friend class RegisterContainer;		// Pakete loeschen, verwalten
friend class Packet;				// Flits gehoeren zu Paket
public: 
	
	//! Konstruktor.
	Flit(Packet* o=NULL, unsigned int i=0, unsigned int m=0);
	
	//! (Virtueller) Konstruktor.
	virtual ~Flit() {}
	
	//! Zugehoeriges Paket.
	/*!
	 * Diese Methode liefert das zugehoerige Paket zum einem Flit.
	 *
	 * @return Das Paket zu dem dieses Flit gehoert.
	 */
	inline const Packet* Owner() const { return this->owner; }

	//! Offset innerhalb des Paketes.
	/*!
	 * Diese Methode liefert die Position eines Flits innerhalb seines
	 * Paketes in Form eines Iterators.
	 *
	 * @return Offset zu diesem Flit innerhalb des Paketes.
	 */
	inline unsigned int Offset() const { return this->offset; }
	
	//! Test auf Head-Flit.
	inline bool head() const
	{
		return this->offset == 0;	
	}

	//! Test auf Tail-Flit.
	inline bool tail() const
	{
		// Ein Tail ist nur ein tail insofern seine Offset-ID mit der
		// maximalen ID uebereinstimmt (packet->size()-1)
		return this->offset == this->maxoffset;
	}
	
	//! Test auf Data-Flit.
	inline bool data() const 
	{
		return !head() && !tail();
	}

	//! Informationen zur aktuellen Position.
	inline const t_storage_info& Location() const { return this->location; }
		
	//! Flit nach Einpufferung aktualisieren.
	virtual void buffered(Buffer* where, signed int since);
	
	//! Flit wurde nicht angenommen.
	virtual void rejected() {}

	//! Test, ob Flit bewegt werden darf.
	virtual bool moveable() const { return true; }
	
	//! Clone des Flits erzeugen
	virtual Flit* clone() const = 0;	

	//! Minimum an verfuegbarem Platz vor Einpufferung.
	virtual unsigned int neededSpace() const; 
	
	//! String-Repraesentation.
	virtual std::string toString() const;

protected:
	
	//! Zugehoeriges Paket setzten.
	/*!
	 * Nach dem Klonen eines Flits ist es erforderlich, dass zugehoerige
	 * Paket bzw. dessen Adresse zu aktualisieren, da geklonte Flits immer 
	 * zu ebenfalls geklonten Paketen gehoeren.
	 *
	 * @param packet Zugehoeriges, geklontes Paket zu diesem Flit.
	 */
	void setOwner(Packet* packet);

	//! Das zugehoerige Nachrichtenpaket.
	Packet* owner;

	//! Position des Flits im Nachrichtenpaket.
	unsigned int offset;

	//! Letzte Position in Nachrichtenpaket.
	unsigned int maxoffset;
	
	//! Information zur aktuellen Position.
	t_storage_info location;
};

#endif
