#ifndef __NODE_H__
#define __NODE_H__

#include <wire.h>

class Node;
class TriggerNode;
class WireMatrixModifier;
class SignedTNodeList;

typedef std::list<WireMatrixModifier*> ModifierList;

typedef std::list<Node*> NodeList;
typedef std::list<TriggerNode*> TNodeList;

typedef std::vector<Node*> NodeVector;
typedef std::vector<TriggerNode*> TNodeVector;

//
// Das ist nicht schoen ):, aber der cast funktioniert seit gcc-4.1 nicht mehr.
//
#define NLITNULL (TNodeList::iterator)0 

//! Die Knoten eines Verbindungsnetzes.
/*!
 * Ein Netz besteht aus Knoten und Kanten. Knoten sind jene Netzelemente, welche 
 * durch (gerichtete) Leitungen bzw. Routen, den Kanten des Netzes, mit wieder 
 * anderen Knoten verbunden sind. Knoten koennen Pakete generieren (Quellen), 
 * speichern (Puffer), entfernen (Ziele) oder vermitteln (Router).
 *
 * @see Wire
 * @see PacketgeneratorElem
 * @see Router
 * @see Buffer
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Node : public Component
{
friend class XmlSimulation;
friend class Simulation;
friend class Test_RegisterContainer;

public:

	//! Rueckgabewerte fuer Senden/Empfangen von Paketen.
	enum t_trigger_return
	{
		ERROR,			// Fehler, unerwarteter Programmzustand
		DISMISS,		// Abbruch, eine Bedingung wurde verletzt
		NOPACKET, 		// Kein Paket zu senden/empfangen
		WAIT, 			// Status noch unklar
		SUCCESS 		// Aktion erfolgreich
	};

	//! Interface fuer Knotenfilter.
	/*!
	 * Ein Filter fuer Knoten kann dazu verwendet werden, die Suche nach
	 * Knoten geeignet einzuschraenken.
	 *
	 * @see Node::prevNodes(NodeFilter*)
	 * @see Node::nextNodes(NodeFilter*)
	 */
	struct Filter
	{
		//! Testmethode.
		/*!
		 * @param node Der zu testende Knoten.
		 * @return true, falls Filterkriterium erfuellt.
		 */
		virtual bool valid(Node* node) const = 0;
		
		//! Destruktor,
		virtual ~Filter() {};
	};

	//! Empfangen eines Flits von Eingangsleitung.
	/*!
	 * Diese Methode wird aufgerufen, wenn ein Knoten ein Flit von einer 
	 * eingehenden Leitung empfangen soll. Im Erfolgsfall (Node::SUCCESS)
	 * wird ein uebergebenes Flag gesetzt, sofern das Ende eines Paketes
	 * empfangen wurde (ansonsten zurueckgesetzt).
	 *
	 * @param inwire Eine eingehende Leitung dieses Knotens.
	 * @param tail Flag fuer Ende eines Paketes. 
	 * @return Status (e_trigger_return).
	 */
	virtual int takePacketFromLine(Wire* inwire, bool& tail) = 0;
	
	//! Versenden eines Flitss ueber Augangsleitung.
	/*!
	 * Diese Methode wird aufgerufen, wenn ueber eine ausgehende Leitung eines
	 * Knotens ein Flit versendet werden soll. Im Erfolgsfall (Node::SUCCESS)
	 * wird ein uebergebenes Flag gesetzt, sofern das erste Flit eines Paketes
	 * angelegt wurde (ansonsten zurueckgesetzt).
	 *
	 * @param outwire Eine ausgehende Leitung dieses Knotens.
	 * @param head Flag fuer Beginn eines Paketes. 
	 * @return Status (e_trigger_return).
	 */
	virtual int putPacketOnLine(Wire* outwire, bool& head) = 0;

	//! Signal fuer Aktivierung einer eingehenden Leitung.
	/*!
	 * Das Versenden von Paketen erfolgt von den Quellen zu den Zielen. Die
	 * Leitungen in einem Verbindungsnetz sind gerichtet, so dass eingende
	 * Leitungen stets aus Richtung der Quellen an einen Knoten angeschlossen
	 * werden. Fuer die Bewegung von Paketen muss zunaechst der Vorgaenger eines
	 * Knotens ein Paket an eine ausgehende Leitung anlegen und seinem Nachfolger
	 * signalisieren, dass ein Paket (oder kein Paket) versendet werden kann.
	 * Die Reaktion auf dieses Signal wird in dieser Methode implementiert. 
	 * Optional kann eine Liste und eine position uebergeben werden, in der
	 * alle moeglichen Empfaenger fuer ein angelegtes Paket zurueckgeliefert
	 * werden.
	 *
	 * @param inwire Eine aktivierte eingende Leitung dieses Knotens.
	 * @param modifiers Eine Liste von Schaltmatrix-Manipulatoren.
	 * @param list Liste fuer moegliche Empfaenger, falls Paket anliegt.
	 * @param pos Einfuegeposition.
	 * @return Status (e_trigger_return).
	 */
	virtual int lineActivated(Wire* inwire, ModifierList* modifiers,
							SignedTNodeList* list=NULL, TNodeList::iterator pos=NLITNULL) = 0;

	//! Signal fuer Abschluss eines Sendevorgangs.
	/*!
	 * Diese Methode signalisiert einem Knoten, dass eine von ihm aktivierte 
	 * ausgehende Leitung von seinem Nachfolgenden Knoten bearbeitet wurde.
	 * Ein Knoten, der gesendet hat, kann nicht noch einmal senden und hat
	 * damit seine maximale Aufnahmekapazitaet in einem Takt erreicht. Wird
	 * (optional) eine Liste und eine Position spezifiziert, so werden alle
	 * Knoten (TriggerNodes), die sich um diesen Knoten beworben haben und
	 * keine weiteren Knoten zur Verfuegung haben, zurueckgeliefert.
	 *
	 * @param outwire Eine fertig bearbeitete ausgehende Leitung dieses Knotens.
	 * @param modifiers Eine Liste von Schaltmatrix-Manipulatoren.
	 * @param list Liste fuer Knoten mit geklaertem Sende-Status.
	 * @param pos Einfuegeposition.
	 * @return Status (e_trigger_return). 
	 */
	virtual int lineFinished(Wire* outwire, ModifierList* modifiers,
							 SignedTNodeList* list=NULL, TNodeList::iterator pos=NLITNULL) = 0;
		
	//! Naechste Netzknoten in Richtung der Ziele.
	/*!
	 * Diese Methode bestimmt anhand der vorliegenden Netzstruktur die naechsten
	 * Knoten in Richtung der Ziele (Nachfolger). Ueber einen Filter kann diese
	 * Suche gesteuert werden. Fuer einen Nachfolger, der die Filter-Bedingung
	 * nicht erfuellt, werden dessen Nachfolger untersucht.
	 *
	 * @see NodeFilter
	 * 
	 * @param nf Ein (optionaler) Filter.
	 * @return Vektor mit (gefilterten) Nachfolgern.
	 */
	NodeVector nextNodes(Node::Filter* nf=NULL) const;

	//! Naechste Netzknoten in Richtung der Quellen.
	/*!
	 * Diese Methode bestimmt anhand der vorliegenden Netzstruktur die vorigen
	 * Knoten aus Richtung der Quellen (Vorgaener). Ueber einen Filter kann diese
	 * Suche gesteuert werden. Fuer einen Vorgaenger, der die Filter-Bedingung
	 * nicht erfuellt, werden dessen Vorgaenger untersucht.
	 *
	 * @see NodeFilter
	 * 
	 * @param nf Ein (optionaler) Filter.
	 * @return Vektor mit (gefilterten) Vorgaengern.
	 */
	NodeVector prevNodes(Node::Filter* nf=NULL) const;

	//! Eingehende Leitungen des Knotens.
	/*!
	 * Lesender Zugriff auf die eingehenden Leiungen dieses Knotens. Neue
	 * Leitungen koennen ueber die Methode addInWire hinzugefuegt werden.
	 *
	 * @return Referenz auf Vektor mit eingehenden Leitungen.
	 */
	inline const WireVector& In() const
	{
		return _in;
	}

	//! Ausgehende Leitungen des Knotens.
	/*!
	 * Lesender Zugriff auf die ausgehenden Leitungen dieses Knotens.
	 * Neue Leitungen koennen ueber die Methode addOutWire() hinzugefuegt
	 * werden.
	 *
	 * @return Referenz auf Vektor mit ausgehenden Leitungen.
	 */
	inline const WireVector& Out() const
	{
		return _out;
	}

	//
	// TODO: Interface fuer Repraesentation einer Position!!
	//
	std::vector<int> position;

	
protected:
  
	//! Hinzufuegen einer eingehenden Leitung.
	/*!
	 * Diese Methode wird verwendet, um eine eingehende Leitung fuer diesen
	 * Knoten zu setzen. Die Reihenfolge mit der Leitungen hinzugefuegt werden
	 * bestimmt deren Eingangs-Index im Kontext dieses Knotens.
	 *
	 * @param inwire Eine eingehende Leitung dieses Knotens.
	 * @return Eingangs-Index der Leitung.
	 */
	virtual unsigned int addInWire(Wire* inwire)
	{
		_in.push_back(inwire);

		return _in.size()-1;
	}

	//! Hinzufuegen einer ausgehenden Leitung.
	/*!
	 * Diese Methode wird verwendet, um eine ausgehende Leitung fuer diesen
	 * Knoten zu setzen. Die Reihenfolge mit der Leitungen hinzugefuegt werden
	 * bestimmt deren Ausgangs-Index im Kontext dieses Knotens.
	 * ueberladen.
	 *
	 * @param outwire Eine ausgehende Leitung dieses Knotens.
	 * @return Ausgangs-Index der Leitung.
	 */
	virtual unsigned int addOutWire(Wire* outwire)
	{
		_out.push_back(outwire);

		return _out.size()-1;
	}
	
	//! Gitter-Koordinaten setzen.
	/*!
	 * Netzknoten, die in Gitterknoten eingebettet sind, uebernehmen deren
	 * Koordinaten fuer ein positionsabhaengiges Routing. Diese Methode parst 
	 * einen String, der die Koordinaten enthaelt. 
	 *
	 * @param coords String mit Koordinaten dieses Knotens.
	 * @return true, falls Koordinaten erfolgreich geparst.
	 */
	bool setCoordinates(const char* coords);
	
	
private:
	
	//! Die Eingangsleitungen eines Knotens.
	WireVector _in;

	//! Die Ausgangsleitungen eines Knotens.
	WireVector _out;
};

#endif
