#ifndef __REGISTERCONTAINER__
#define __REGISTERCONTAINER__

#include <node.h>

class BufferQueue;

//! Klasse fuer Paketvermittelung aus einer Puffer-Queue.
/*!
 * Pakete bzw. ihre Bestandteile, die Flits, werden auf ihrem Weg durch
 * ein Kommunikationsnetz in den Puffern des Netzes zwischengespeichert.
 * Ein normale Queue erlaubt in der Regel nur Zugriffe nach dem FIFO-Prinzip,
 * was im Falle von (partiellem) Multicast zu unnoetig langen Verzoegerungen
 * fuehrt, da erst auf ein Flit zugegriffen werden kann, wenn alle zuvor
 * eingepufferten Flits vollstaendig versendet worden sind. Diese Klasse
 * ermoeglicht den simultanen Zugriff mehrerer Empfaenger auf eine Queue,
 * indem eingehende Flits und meogliche Empfaenger in internen Strukturen
 * verwaltet und die Queue jeweils nach vollstaendiger Vermittlung des ersten
 * Flits entsprechend modifiziert wird. Empfaenger koennen sich jeweils fuer 
 * das vorderste Paket in der Queue registrieren, auch wenn noch nicht
 * alle zugehoerigen Flits eingepuffert wurden. Ein Paket muss erst vollstaendig
 * vermittelt werden, bevor auf ein eventuell folgendes Paket zugegriffen
 * werden kann.
 * 
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class RegisterContainer
{
public:
	
	//! Konstruktor
	/*!
	 * Der Aufruf dieses Konstruktors erzeugt eine Verwaltungstrukur
	 * zur Vermittlung der Flits der uebergebenen Puffer-Queue.
	 */
	RegisterContainer() : packet(NULL) {}
	
	//! Setzt die Konfiguration zurck.
	/*!
	 * Die internen Verwaltungsinformationen werden auf den initialen 
	 * Zustand zurueckgesetzt.
	 */
	void reset();
	
	//! Ankommendes Flit in Verwaltungsstruktur registrieren.
	/*!
	 * Ein Flit, welches in die zugehoerige Puffer-Queue aufgenommen wird (push),
	 * wird ueber diese Methode, beim Register registriert. Handelt es sich
	 * um den Beginn eines Paketes (head) und es befindet sich gerade kein Paket 
	 * in Vermittelung, so wird das zugehoerige Paket zum aktuell vermittelten
	 * Paket.
	 *
	 * @param flit Neues Flit in zugehoeriger Puffer-Queue.
	 */
	void announceFlit(Flit* flit);

	//! Vermitteltes Flit aus Verwaltungsstruktur entfernen.
	/*!
	 * Wurde ein Flit (das vorderste) vollstaendig vermittelt so kann aus aus der 
	 * zugehoerigen Puffer-Queue entfernt werden (pop). Diese Methode sorgt dafuer,
	 * dass es auch aus der internen Verwaltungsstruktur entfernt wird. Handelt
	 * es sich um das Endes eines Paketes (tail), so wird auch das aktuell zu
	 * vermittelnde Paket aktualisiert und die Registrierung empfangsbereiter
	 * Knoten zurueckgesetzt.
	 *
	 * @param flit Vollstaendig vermitteltes Flit der zugehoerigen Puffer-Queue.
	 */
	void eraseFlit(Flit* flit);

	//! Empfaenger fuer vermittelbares Paket registrieren.
	/*!
	 * Bevor ein Neztknoten ein Flit der zugehoerigen Queue empfangen kann
	 * kann, muss er fuer das zugehoerige Paket registriert werden. Die 
	 * Registrierung sorgt dafuer, dass ein Klon des zu vermittelnden Paketes
	 * fuer diesen Empfaenger generiert wird und die erreichbaren Ziele aufgenommen 
	 * werden. Die Registrierung wird durchgefuehrt, wenn durch Aufruf von 
	 * Node::lineActivated empfangsbereite Knoten (ggf. durch ein Routing) zu 
	 * einem Ausgang eines Puffer-Knotens ermittelt werden. Neben dem Knoten
	 * wird auch ein Bitmaske mit (ggf. durch Routing bestimmten) zulaessigen
	 * Zielen spezifiziert, die bei der Vermittlung beruecksichtigt wird.
	 *
	 * @param customer Ein moeglicher Empfaenger fuer vermittelbares Paket.
	 * @param targets Ueber Empfaenger erreichbare und zulaessige Ziele.
	 */
	void registerCustomer(Node* customer, Bitmask* targets);

	//! Empfaenger fuer Paket entfernen.
	/*!
	 * Wird im Falle von Redundanz ein Paket an einem Router vollstaeandig 
	 * vermittelt, so werden ueber diese Methode Registrierungen redundanter 
	 * Empfaenger entfernt.
	 * 
	 * @param customer Redundanter, nicht mehr gueltiger Empfaenger.
	 */
	void unregisterCustomer(Node* customer);
	
	//! Flit zurueckweiesen.
	/*!
	 * Kann ein Empfaenger ein fuer ihn registriertes Flit nicht entgegennehmen,
	 * so wird dies durch Aufruf dieser Methode dem Register mitgeteilt. Der 
	 * Aufruf sorgt auch dafuer, dass ein entsprechendes Flit in Vorbereitung 
	 * auf die Vermittelung geklont wird.
	 * 
	 * @param customer Registrierter Empfaenger fuer Flit.
	 */
	void rejectFlit(Node* customer);
	
	//! Zugriff auf abholbereites Flit. 
    /*! 
	 * Bevor ein Flit tatsaechlich empfangen wird, kann mit dieser Methode das
	 * abholbereite Flit analysiert werden bzw. geprueft werden, ob ein Flit
	 * empfangen werden kann. Ein solcher Test muss stets vor dem Aufruf von
	 * RegisterContainer::fetchFlit erfolgen. Ist ein Flit verfuegbar, so kann
	 * es sich entweder um das tatsaechlich zu empfangene Flit (Klon), als auch 
	 * um dessen Vorlage (Original) handeln, d.h. bestimmte Information, wie
	 * z.B. das zugehoerige Paket, koennen sich von denen des spaeter empfangenen 
	 * Flits unterscheiden.
	 * 
	 * @param customer Moeglicher (registrierter) Empfaenger fuer ein Flit.
	 * @return Zeiger auf abholbereites Flit oder NULL, falls kein Flit fuer
	 * Empfaenger verfuegbar.
	 */
	const Flit* fetchableFlit(Node* customer) const;

	//! Empfangen eines Flits.
	/*! 
	 * Diese Methode realisiert das Abholen eines Flits durch einen registrierten
	 * Empfaenger, wobei vorausgesetzt wird, dass ein Flit verfuegbar ist. Ein
	 * Flit kann nur einmal von einem Knoten empfangen werden und ein Knoten darf
	 * nur einmal pro Takt empfangen (oder zurueckweisen). Ein Flit, welches das 
	 * Register verlaesst ist stets ein Klon eines entsprechenden Flits des 
	 * vermittelten Paketes. Ein Knoten kann solange Flits empfangen, bis er ein 
	 * Paket vollstaendig empfangen hat. Bei jeder Vermittelung eines Flits werden 
	 * die verbleibenden Ziele (in Form einer Bitamaske) aktualisiert, sind keine 
	 * Ziele mehr offen, so wurde ein Flit (das vorderste) komplett vermittelt und 
	 * kann aus der zugehoerigen Puffer-Queue entfernt werden (pop). Handelt es sich 
	 * um das letzte Flit eines Paketes (tail), so wurde dieses vollstaendig versendet.
	 * In diesem Fall werden alle Registrierungen der Empfaenger zurueckgesetzt, das 
	 * vermittelte Paket (Original) entfernt und das naechste Paket kann versendet 
	 * werden.
	 * 
	 * PRE: Knoten registriert
	 * PRE: Knoten im aktuellen Takt noch nicht empfangen
	 * PRE: Flit ist verfuegbar
	 * POST: Knoten verweist auf naechstes Flit (sofern vorhanden)
	 * 
	 * @param customer Empfangsbereiter (registrierter) Knoten.
	 * @return Klon eines Flits des in Vermittlung befindlichen Paketes. 
	 */
	Flit* fetchFlit(Node* customer);

	//! Vom Register verwaltetes Paket.
	/*!
	 * Der Aufruf dieser Methode leifert das aktuell vermittelte Paket des
	 * Registers, sofern eines vorhanden.
	 *
	 * @return Zu vermittelndes Paket oder NULL, falls keines vorhanden.
	 */
	const Packet* getPacket() const { return packet;};

	//! Vorderstes Flit im Container.
	/*!
	 * Alle Flits, die in die Queue zu diesem Register aufgenommen werden,
	 * werden in einer internen Liste dieses Registers mit zusaetzlichen
	 * Informationen verwaltet. Ist mindestens ein Flit registriert, so 
	 * liefert diese Methode das erste Flit der Liste ansonsten NULL.
	 * 
	 * @return RegisterContainer::flitlist.front() oder NULL.
	 */
	const Flit* firstFlit() const;
	
	//! Container fertig bearbeitet.
	/*!
	 * Diese Methode bestimmt, ob sich der Zustand des Registers bzw. der 
	 * zugehoerigen Puffer-Queue im aktuellen Takt noch aendern wird oder den 
	 * finalen Zustand erreicht hat. Der finale Zustand liegt vor, wenn alle
	 * registrierten Empfaenger fuer das aktuelle Paket entweder ihre Flits abgeholt
	 * bzw. zurueckgewiesen oder bereits das Paket vollstaendig empfangen haben. 
	 * Zur Auswertung dient eine interne Zaehlerstruktur.
	 *
	 * @return true, falls Zustand des Registers fuer aktuellen Takt geklaert.
	 */
	bool finalState();

	//! Ueber registrierten Empfaenger erreichbare und zulaessige Ziele.
	/*!
	 * Bei der Registrierung eines Knoten als Empfaneger fuer das aktuelle Paket
	 * wird eine Bitmaske mit den zulaessigen, der ueber den Knotens erreichbaren,
	 * Zielen uebergeben, die entweder duch das Routing oder von den tatsaechlich 
	 * (durch Netzaufbau) erreichbaren Zielen bestimmt wird. Von diesen Zielen
	 * werden spaeter noch die Ziele abgezogen, die bereits (bei Vermittlung des
	 * Heads) von anderen Knoten bedient wurden. Diese Methode erlaubt den Zugriff 
	 * auf diese Bitmaske zulaessiger/bedienter Ziele eines registrierten Knotens.
	 *
	 * @param customer (Registrierter) Puffer-Knoten.
	 * @return Bitamake mit erreichbaren/zulaessigen Zielen ueber spezifizierten Knoten
	 * oder NULL, falls Knoten nicht registriert.
	 */
	const Bitmask* validTargets(Node* customer);

private:

	//! Verwaltungsstruktur fuer Flit im Register.
	struct FlitInfo
	{
		//! Konstruktor.
		FlitInfo(Flit* f, const Bitmask& mask) : flit(f) { remain = new Bitmask(mask); }
			
		//! Registriertes Flit.
		Flit* flit;
		
		//! Verbleibende Ziele.
		Bitmask* remain;
	};

	//! Verwaltungsstruktur fuer Abnehmer eines Flits aus diesem Register.
	struct CustomerInfo
	{
		//! Default-Konstruktor.
		CustomerInfo() 
		: valid(false), offset(0), packet(NULL), customer(NULL), reachable(NULL) {}
				
		//! Zuruecksetzen der Strukur nach versenden eines Paketes.
		void reset();
		
		//! Markierung fuer Gueltigkeit dieser Struktur. 
		bool valid;

		//! Position des aktuellen Flits in Verwaltungsstruktur.
		std::list<FlitInfo>::iterator offset;
		
		//! Paket welches gerade an Empfaenger versendet wird.
		Packet* packet;

		//! Markierung fuer abgeholtes Flit im aktuellen Takt.
		Signature fetched;

		//! Markierung fuer zurueckgewiesenes Flit im aktuellen Takt.
		Signature rejected;
		
		//! Empfaenger.
		Node* customer;

		//! Ueber Empfaenger erreichbare Ziele.
		Bitmask* reachable;
	};

	//! Zaehlerstruktur fuer taktbezogenen Status dieses Registers.
	struct Counter
	{
		//! Default-Konstruktor.
		Counter() : registered(0), rejected(0), fetched(0) {};

		//! 
		void refresh();
		
		//! Initialen Zustand wiederherstellen.
		void reset();
		
		//! Markierung fuer Zuruecksetzen zu Taktbeginn.
		Signature refreshed;

		//! Anzahl registrierter Empfaenger.
		unsigned int registered;
		//! Anzahl registrierter Empfaenger, die nicht empfangen konnten.
		unsigned int rejected;
		//! Anzahl registrierter Empfaenger, die empfangen haben.
		unsigned int fetched;
	
	} counter;
	
	//! Informationen zu registrierten Flits.
	std::list<FlitInfo> flitlist;

	//! Informationen zu registrierten Empfaengern.
	std::vector<CustomerInfo> customers;

	//! Aktuell zu vermittelndes Paket.
	Packet *packet;
};
	
#endif

