#ifndef __SIGNATURE_H__
#define __SIGNATURE_H__

#include <limits.h>


//! Signatur der Simulation.
/*!
 * Eine Signatur bietet die Moeglichkeit, Komponenten einer Simulation
 * fuer einen Takt zu signieren, so dass bestimmte Aktionen nur einmal
 * durchgefuehrt werden oder die Gueltigkeit von Status-Informationen
 * geprueft werden kann. Fuer jeden neuen Takt werden alle Signaturen 
 * annuliert. 
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Signature 
{
// Signaturen werden am Taktende annuliert
friend class Simulation;
friend class Test_RegisterContainer;

public:
	
	//! Konstruktor.
	/*!
	 * Die Signatur wird initial mit einem ungueltigen Schluessel
	 * versehen.
	 */
	Signature()
	{ 
		invalidate(); 
	}
    		
	//! Signatur aktualisieren.
	/*!
	 * Der Schluessel wird auf den Referenzschluessel gesetzt, wordurch 
	 * die Signatur gueltig wird.
	 */
	inline void sign()
	{
		_key = _reference;
	}
    
	//! Signatur annulieren.
	/*!
	 * Der Schluessel wird auf einen Wert gesetzt, der die Signatur
	 * ungueltig macht.
	 */
	inline void invalidate()
	{
		_key = _reference ? _reference-1 : UINT_MAX;
	}
	
	//! Signatur pruefen.
	/*!
	 * Eine Signatur ist ganau dann gueltig, wenn der Schluessel mit 
	 * dem Referenzschluessel uebereinstimmt.
	 * 
	 * @return true, falls Signatur gueltig.
	 */
	inline bool valid() const 
	{ 
		return (_key == _reference); 
	}
	 
	/*!
	 * Der Schluessel der Signatur.
	 *
	 * @return Aktueller Schluessel der Signatur.
	 */
	inline unsigned int Key() const 
	{ 
		return _key; 
	}
 
	/*!
	 * Der Referenzschluessel der Signatur.
	 * 
	 * @return Aktueller Referenzschluessel der Signatur.
	 */
	inline unsigned int Reference() const
	{
		return _reference;
	}	

	
protected:
  
	//! Annulieren aller Signaturen. 
	/*!
	 * Die Gueltigkeit aller Signaturen haengt vom Referenzschluessel
	 * ab. Diese Methode inkrementiert diese Referenz und annuliert
	 * damit alle Schluessel.
	 */
	inline static void InvalidateAll()
	{
		_reference++;
	}
	
private:
	
	//! Der Schluessel der Signatur.
	unsigned int _key;

	//! Referenzschluessel aller Signaturen.
	static unsigned int _reference;
};

#endif
