#ifndef __SOURCEBUFFER_H__
#define __SOURCEBUFFER_H__

#include <buffer.h>

class Packet;
class Switching;
class Distribution;
class TrafficDistribution;

//! Die Quellen eins Verbindungsnetzes (Sourcen).
/*!
 * Die Quellen eines Verbindungsnetzes sind spezielle Puffer, die ihre 
 * Flits nicht von Vorgaengerknoten erhalten, sondern Pakete, ueber eine
 * zeitliche Verteilung festgelegt, selbst generieren. Fuer die generierten
 * Pakete werden sowohl Anzahl (Multicast), als auch die Identifier der
 * zu erreichenden Ziele, sowie die Prioritaet ebenfalls ueber eigens dafuer 
 * vorgesehene Verteilungen bestimmt. Die Queue einer Quelle enthaelt 
 * immer genau ein Paket. Gelangt ein Paket nach seiner Generierung nicht 
 * ins Netz so wird es verworfen. 
 *
 * @see Buffer
 * @see Traffic
 * @see Distribution
 * 
 * An einer Quelle kann der Durchsatz an Flits gemessen werden.
 *
 * @see Analysable
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class SourceBuffer: public Buffer 
{
public:	

	//! Konstruktor.
 	/*!
	 * Es erfolgen lediglich notwendige Initialisierungen der Klassenattribute.
	 * In seiner Eigenschaft als XMLElement kann ein zugehoeriges DOMElement
	 * fuer die Quelle spezifiziert werden. Der Abschluss der Initialisierung
	 * einer Quelle erfolgt erst durch Aufruf der Methode init.
	 *
	 * @see XMLElement
	 *
	 * @param scope Zugehoeriges DOMElement mit Beschreibung dieser Quelle.
	 */
	SourceBuffer(DOMElement* scope=NULL);
    
    // Implementierung der Basisklasse XMLElemet.
    virtual bool init(const DOMElement* scope=NULL);

    /*! 
	 * Implemetierung der Basisklasse Component.
	 *
	 * @see Component
	 * @return Component::SOURCE
	 */
    virtual const int rtti() const 
	{
		return Component::SOURCE; 
	}

	/*! 
	 * Implementierung der Basisklasse Component.
	 *
	 * @see Component
	 * @return "Source"
	 */
	const std::string Name() const 
	{ 
		return std::string ("Source"); 
	} 
	
	//! Implementierung der Basisklasse Component.
	/*!
	 * Die interne Queue wird geleert und die zeitliche Paketverteilung
	 * zurueckgesetzt (Multicast, Prioritaten und lokaler Verkehr sind
	 * gedaechtnislos).
	 * 
	 * @see Component
	 */
	virtual void reset();
    
	//! Implementierung der Basisklasse Analysable.
	/*!
	 * Die einzige Messung fuer Quellen ist derzeit der Durchsatz an
	 * Flits. Dazu wird geprueft, ob ein Flit im aktuellen Takt die 
	 * Queue verlassen hat.
	 * 
	 * @see BufferQueue::Popped
	 * @see Analysable
	 */
    virtual double Measure(TypeInfo type);

	//! Implementierung der Basisklasse Analysable.
	/*!
	 * Liegt fuer ueber erstes Argument sezifizierte Messung ein neuer
	 * Messwert vor, so wird Messung angestossen. Derzeit gibt es fuer
	 * Quellen nur eine Messung fuer den Durchsatz, fuer die immer ein
	 * Messwert vorliegt.
	 * 
	 * @see Analysable
	 */
	virtual void takeMeasure(const Description& descr);   
	
	//! Implementierung der Basisklasse Node.
	/*!
	 * Ein Aufruf dieser Methode sorgt dafuer, dass ein Puffer versucht, an 
	 * seine ausgehende Leitung ein Paket anzulegen. Im Falle einer Quelle
	 * wird dazu in Abhaengigkeit von der zeitlichen Verteilung ein neues 
	 * Paket generiert.
	 * 
	 * @see Buffer
	 * @see Node
	 */
	virtual int putPacketOnLine(Wire* w, bool& head);

	//! Implementierung der Basisklasse Buffer.
	/*!
	 * Im Gegensatz zur Basisklasse sind keine Messungen zu Delay und
	 * Latency notwendig. Es muss allerdings festgehalten werden, ob ein
	 * Paket ins Netz gelangt ist, d.h. im Falle von (partiellem) Multicast
	 * der Head an mindestens einen Empfaenger versendet wurde. Ist
	 * dies nicht der Fall, so wird ein Paket verworfen.
	 *
	 * @see Buffer
	 */
	virtual void flitTransmitted(Flit* flit, bool complete);
	
    //! Informationen zu erreichbaren Zielen aktualisieren.
	/*!
	 * Zu Beginn einer Simulation muessen alle Quellen ihre Informationen
	 * zu erreichbaren Zielen aktualisieren. Dazu wird die rekursive 
	 * Initialisierung aller Bitmasken im Netz angestossen und anschliessend
	 * geprueft, welche Ziele von dieser Quelle aus erreichbar sind. Mit 
	 * diesen Information werden dann die Verteilungen fuer Multicast und 
	 * lokalem Verkehr initialisiert.
	 * 
	 * @see Component::generateBitmask
	 */
    void checkReachableTargets();

protected:
	
    //! Ein Paket generieren.
    /*!
     * Wird der Generator aktiviert und es liegt kein Paket, bzw. dessen
     * Fragmente, in der lokalen Queue, so wird ueber diese Methode ein
     * neues Paket generiert und eingepuffert.
     *
     * @return true, falls neues Paket generiert wurde.
     */
    bool generatePacket();

private:
    
    //! Multicast-Verteilung
    Distribution *mc_dist;
    //! Lokaler Verkehr
    Distribution *lc_dist;
    //! Paketprioritaeten
    Distribution *pr_dist;
    //! Zeitliche Paketverteilung
    TrafficDistribution *tr_dist;
	
    //! Eine Liste der erreichbaren Ziele.
    /*!
     * Diese Liste beinhaltet alle Numern der vom Generator aus erreichbaren Ziele.
     * Anwendung findet diese Funktion bei der Auswahl der/des Zeile(s) eines Paketes.
     * Es kann so kein Paket generiert werden, dessen Ziel nicht erreichbar ist.
     */
    std::vector<unsigned int> reachableTargets;
     
	// konstante Packetgroesse, falls gewuenscht
    // TODO: Verteilungen, Zufallsgroessen
    unsigned int constPacketSize;   
    
	//! Alle Pakete haben diese Anzahl Takte Zeit, ihr Ziel zu erreichen.
    unsigned int deadline;
    
    //! Hoechste Paket-Prioritaet fuer diesen Generator.
    unsigned int maxPriority;

	//! Flag fuer (unvollstaendig) ins Netz gelangtes Paket.
	/*!
	 * Im Falle von (partiellem) Multicast ist das Kriterium, wonach ein Paket ins 
	 * Netz gelangt ist, wenn der Head die Queue verlassen hat, nicht ausreichend.
	 * Dieses Flag wird gesetzt, wenn der Head eines Paketes an mindestens einen
	 * Empfaenger vermittelt wurde.
	 */
	bool p_accepted;
};

#endif
