#ifndef __TRIGGERNODE_H__
#define __TRIGGERNODE_H__

#include <node.h>
#include <signature.h>


//! Aktivierungsknoten eines Netzes. 
/*!
 * Diese abstrakte Klasse beschreibt all die Knoten eines Verbindungsnetzes,
 * die von einer Koroutine, die fuer die Paketbewegung waehrend eines Taktes
 * sorgt, angesteuert (getriggert) werden. Ein solcher Knoten kann aktiviert 
 * und zum Senden oder Empfangen veranlasst werden. 
 * 
 * @see PacketForwarding
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class TriggerNode : public Node
{

public:
		
	//! Status-Information.
	enum e_trigger_info
	{
		ACTIVE,		// von Koroutine aktiviert 
		PULLED,		// Senden abgeschlossen (oder nicht moeglich) 
		PUSHED,		// Empfangen abgeschlossen (oder nicht moeglich)
		EOTI		// Ende der Aufzaehlung
	};

	//! Filter fuer TriggerNodes.
	struct Filter : public Node::Filter
	{
		//! Implementierung der abstrakten Struktur Node::Filter.
		/*!
		 * Alle Netzknoten ausser Router sind TriggerNodes.
		 * 
		 * @return true, falls Knoten kein Router.
		 */
		bool valid(Node* found) const
		{
			return found->rtti() != Component::ROUTER;
		}
	};
		
	//! Hinzufuegen eines Nachfolgers fuer Aktivierung.
	/*!
	 * Eine Koroutine bearbeitet eine Netzstruktur in einer festgelgten Reihenfolge
	 * ihrer Knoten. Dazu verwaltet ein jede Instanz dieser Klasse einen Vektor
	 * von Aktivierungsknoten, die im Ablauf der verwendeten Koroutine als naechstes 
	 * bearbeitet werden sollen.
	 * 
	 * @param child Ein Nachfolger aus Sicht der Koroutine.
	 */
	inline void addChild(TriggerNode* child) 
	{ 
		_children.push_back(child); 
	}

	//! Zugriff auf alle Nachfolger.
	/*!
	 * Diese Methode bietet Zugriff auf alle aus Sicht der Koroutine nachfolgend
	 * zu aktivierenden Knoten.
	 * 
	 * @return Vektor aller nachfolgend zu aktivierenden Knoten.
	 */
	inline const TNodeVector& children() const 
	{ 
		return _children; 
	}

	//! Test auf vorhandene Nachfolger.
	/*!
	 * Diese Methode prueft, ob aus Sicht der verwendeten Koroutine nachfolgende
	 * Knoten vorhanden sind.
	 * 
	 * @return true, falls Nachfolger vorhanden.
	 */
	inline bool hasChildren() const 
	{ 
		return !_children.empty(); 
	}
	 
	/*!
	 * Trigger-Information setzen.
	 *
	 * @see e_trigger_info
	 * @param info Zu setzende Information.
	 */
	inline void set(e_trigger_info info)
	{
		assert(info < EOTI);
		
		_signatures[info].sign();
	}
	
	/*! 
	 * Trigger-Information aufheben.
	 *
	 * @see e_trigger_info
	 * @param info Aufzuhebende Information.
	 */
	inline void unset(e_trigger_info info)
	{
		assert(info < EOTI);
	
		_signatures[info].invalidate();
	}
	
	/*! 
	 * Trigger-Information ueberpreufen.
	 *
	 * @see e_trigger_info
	 * @param info Zu ueberpruefende Information.
	 */
	inline bool test(e_trigger_info info) const
	{
		assert(info < EOTI);

		return _signatures[info].valid();
	}
		
private:
	
	//! Status-Informationen.
	Signature _signatures[EOTI];
	
	//! Nachfolgende Knoten aus Sicht der Koroutine.
	TNodeVector _children;

};


//! Liste signierter Knoten.
/*!
 * Diese Klasse implementiert eine Liste von TriggerNodes, die beim 
 * Hinzufuegen von Knoten eine Signatur setzt. Bereits signierte
 * Knoten werden NICHT hinzugefuegt.
 *
 * @see TriggerNode::e_trigger_info
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class SignedTNodeList : public TNodeList
{
public:
	
	//! Konstruktor.
	SignedTNodeList(TriggerNode::e_trigger_info info=TriggerNode::EOTI) 
		: _info(info) {}
	
	/*!
	 * Einfugen an bestimmter Position.
	 *
	 * @param pos Position in Liste, an der Knoten eingefuegt werden soll.
	 * @param tn Einzufuegender (unsignierter) Knoten.
	 * @return Iterator zu eingefuegtem Knoten oder pos, falls Knoten markiert.
	 */
	TNodeList::iterator insert(TNodeList::iterator pos, TriggerNode* tn)
	{
		if(check(tn))
			return TNodeList::insert(pos,sign(tn));
		return 
			pos;
	}

	/*!
	 * Einfuegen am Ende der Liste.
	 *
	 * @param tn Einzufuegender (unsignierter) Knoten.
	 */
	void push_back(TriggerNode* tn)
	{
		if(check(tn))
			TNodeList::push_back(sign(tn));
	}

	/*!
	 * Einfugen am Anfang der Liste.
	 *
	 * @param tn Einzufuegender (unsignierter) Knoten.
	 */
	void push_front(TriggerNode* tn)
	{
		if(check(tn))
			TNodeList::push_front(sign(tn));
	}

protected:

	//! Knoten signieren.
	inline TriggerNode* sign(TriggerNode* tn)
	{
		if(_info < TriggerNode::EOTI) tn->set(_info);
		return tn;
	}

	//! Test, ob Knoten bereits signiert. 
	inline bool check(TriggerNode* tn)
	{
		return _info == TriggerNode::EOTI || !tn->test(_info);
	}
	
	//! Markierungsfeld einzufuegender Knoten.
	TriggerNode::e_trigger_info _info;
};

#endif
