#ifndef __CONFIGDIRECTIVES_H__
#define __CONFIGDIRECTIVES_H__

#include <utility/valuestring.h>
#include <utility/hashmap.h>
#include <config.h>

#include <string>
#include <vector>

typedef StringMap<ValueString> Directives;

using std::string;
using std::vector;
using std::ostream;

//! Verwaltung von Konfigurationsinformationen.
/*!
 * Diese Klasse realisiert die Verwaltung von Informationen in Form 
 * von (Key,Value) Paaren.
 *
 * @see ValueString
 *
 * @author Arvid Walter <asas@cs.tu-berlin.de>
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class ConfigDirectives 
{

public:

	//! Konstruktor.
	ConfigDirectives();

	//! Copy-Konstruktor.
	ConfigDirectives(const ConfigDirectives &rhs);

	//! (Virtueller) Destruktur.
	virtual ~ConfigDirectives();
	
	//! Zuweisungs-Operator.
	/*!
	 * Der Zuweisungsoperator ersetzt die (Key,Value) Paare des ersten
	 * Argumentes vollstaendig durch die des zweiten Argumentes.
	 */
	ConfigDirectives& operator=(const ConfigDirectives& rhs);
	
	//! Aktualsierungs-Operator.
	/*!
	 * Dieser Operator fuegt dem ersten Argument alle (Key,Value) Paare
	 * des zweiten Argumentes hinzu, wobei vorhandene Eintraege ersetzt,
	 * nicht vorhandene Eintraege neu eingefuegt werden.  
	 */
	ConfigDirectives& operator<<=(const ConfigDirectives& rhs);
	
	//! Vervollstaendigungs-Operator.
	/*!
	 * Dieser Operator fuegt dem ersten Argument alle (Key,Value) Paare
	 * des zweiten Argumentes hinzu, zu denen bisher noch kein Key 
	 * existiert.
	 */
	ConfigDirectives& operator+=(const ConfigDirectives& rhs);
	
	//! Aktualsierung von Konfigurationsinformationen.
	/*!
	 * Die Konfigurationsinformationen werden mit den Werten dieser 
	 * Konfiguration aktualisiert. 
	 *
	 * @param config Zu aktualisierende Konfigurationsinformationen.
	 */
	inline void updateConfig(ConfigDirectives* config) const
	{
		(*config) <<= (*this);
	}

	//! Vervollstaendigung von Konfigurationsinformationen.
	/*!
	 * Fehlende Konfigurationsinformationen werden mit Werten dieser
	 * Konfiguration vervollstaendigt.
	 *
	 * @param conf Zu vervollstaendigende Konfigurationsinformationen.
	 */
	inline void completeConfig(ConfigDirectives* config) const
	{
		(*config) += (*this);
	}
		
	//! Ersetzen von Konfigurationsinformationen.
	/*!
	 * Die Konfigurationsinformationen werden vollstaendig durch Werte dieser
	 * Konfiguration ersetzt.
	 *
	 * @param conf Zu ersetztende Konfigurationsinformationen.
	 */
	inline void replaceConfig(ConfigDirectives* config) const
	{
		(*config) = (*this);
	}

	//! Eintrag hinzufuegen.
	/*!
	 * Ein neuer Eintrag wird hinzugefuegt oder bestehender Eintrag mit
	 * gleichem Schluessel ersetzt.
	 * 
	 * @param key Identifier.
	 * @param value Wert fuer Identifier.
	 */
	virtual void addValue(string key, ValueString value);
	
	//! Eintrag entfernen.
	/*!
	 * Eintrag zu Identifier key wird entfernt. Existiert kein Eintrag,
	 * so bleibt Konfiguration unveraendert.
	 * 
	 * @param key Identifier.
	 */
	virtual void removeValue(const string& key);
	
	//! Konfiguration verwerfen.
	/*!
	 * Alle Eintraege der Konfiguration werden entfernt.
	 */
	virtual void clear();
	
	//! Lesender Zugriff.
	/*!
	 * Liefert Kopie des Eintrages zu Identifier key, oder leeres Objekt,
	 * falls kein Eintrag vorhanden.
	 *
	 * @param key Identifier.
	 * @return Wert zu Identifier (Kopie).
	 */
	virtual ValueString getValue(const string& key) const;
	
	//! Schreibender Zugriff.
	/*!
	 * Liefert Referenz auf Eintrag zu Identifier Key. Existiert kein 
	 * Eintrag, so wird ein neuer Eintrag mit leerem Objekt erzeugt.
	 *
	 * @param key Identifier.
	 * @return Wert zu Identifier.
	 */
	virtual ValueString& operator[](const string& key);
	
	//! Liste aller Identifier der Konfiguration.
	/*!
	 * Liefert List aller aktuellen Identifier dieser Konfiguration.
	 * 
	 * @return Liste mit aktuell vorhandenen Identifiern.
	 */
	virtual vector<string> getKeys() const;
	
	//! Test auf vorhandenen Eintrag.
	/*!
	 * Es wird geprueft, ob ein Eintrag zu Identifier key in dieser
	 * Konfiguration existiert.
	 * 
	 * @return true, falls Eintrag zu Identifier key vorhanden.
	 */
	virtual bool isSet(const string& key) const;
	
	//! Test auf leere Konfiguration.
	/*!
	 * Es wird geprueft, ob diese Konfiguration ueber keinen Eintrag 
	 * verfuegt.
	 * 
	 * @return true, falls Konfiguration leer.
	 */
	virtual bool isEmpty() const;
	
	//! Ausgabe in Stream.
	/*!
	 * Alle Eintrage werden formatiert ausgegeben.
	 * 
	 * @param out Der Stream fuer die Ausgabe.
	 */
	virtual void print(ostream& out) const;
			
	//! Ausgabe nach stdout.
	virtual void print() const;

protected:
	
	//! Lokale Konfiguration.
	Directives directives;
};

ostream & operator<<(ostream &out, ConfigDirectives &rhs);


#endif
