#include <sys/time.h>


//! Zeitmessung in Sekunden, Millisekunden und Mikrosekunden.
/*!
 * Diese Klasse realisiert eine einfache Zeitmessung (Dauer) in Form von Sekunden,
 * Millisekunden und Mikrosekunden. Ausgabe der gemessenen Zeitdauer kann fortlaufend
 * erfolgen oder Start- und Endzeitpunkt explizit gesetzt werden.
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class Timekeeper
{
public:

	//! Konstruktor.
	/*!
	 * Die Zeitmessung wird mit aktueller Zeit als Startzeitpunkt initialisiert.
	 *
	 * @see reset()
	 * @see start()
	 */
	Timekeeper()
	{
		reset();
		start();
	}
	
	//! Start der Zeitmessung.
	/*!
	 * Der Startzeitpunkt fuer eine Zeitmessung kann ueber diese Methode
	 * explizit gesetzt werden, anderenfalls wird der Zeitpunkt des
	 * Konstruktoraufrufs als Referenz verwendet.
	 */
	inline void start()
	{
		gettimeofday(&_start_time, (struct timezone*)0);	
	}

	//! Ende der Zeitmessung.
	/*!
	 * Das Ende der Zeitmessung kann durch Aufruf dieser Methode explizit 
	 * gesetzt werden, anderenfalls werden fortlaufende Messungen in Relation 
	 * zum (festen) Startzeitpunkt durchgefhrt.
	 *
	 * @see start()
	 */
	inline void end()
	{
		_finished = true;
		
		gettimeofday(&_end_time, (struct timezone*)0);
	}

	//! Interne Datenstrukturen zuruecksetzen.
	/*!
	 * Die internen Strukturen zur Verwaltung einer Zeitmessung werden
	 * zurueckgesetzt, der Startzeitpunkt bleibt unveraendert.
	 *
	 * @see end()
	 * @see start()
	 */
	inline void reset()
	{
		_finished = false;
	}
	
	//! Zeitdauer in Mikrosekunden.
	/*!
	 * Die gemessene Zeitdauer in Mikrosekunden, als Differenz zwischen Start 
	 * und Ende der Messung, wird wiedergegeben. Start- und Endzeitpunkt koennen
	 * zuvor explizit angegeben werden.
	 *
	 * @see start()
	 * @see end()
	 * @return Zeit zwischen Start und Ende der Messung in Mikrosekunden. 
	 */
	double micros()
	{
		if(!_finished)
		{
			gettimeofday(&_end_time, (struct timezone*)0);	
		}
			
		return (double)(_end_time.tv_sec - _start_time.tv_sec) * 1000000.0 +
                 (_end_time.tv_usec - _start_time.tv_usec);	
	}

	//! Zeitdauer in Millisekunden.
	/*!
	 * Umrechnung der gemessenen Zeitdauer von in Millisekunden.
	 * 
	 * @see micros()
	 * @return Zeit zwischen Start und Ende der Messung in Millisekunden. 
	 */
	inline double millis()
	{
		return micros() / 1000.0;
	}

	//! Zeitdauer in Sekunden.
	/*!
	 * Umrechnung der gemessenen Zeitdauer in Sekunden.
	 * 
	 * @see micros()
	 * @return Zeit zwischen Start und Ende der Messung in Sekunden.
	 */
	inline double secs()
	{
		return millis() / 1000.0;
	}

	
protected:

	//! Flag fuer Ender der Messung.
	bool _finished;
	//! Startzeitpunkt.
	struct timeval _start_time;
	//! Endzeitpunkt.
	struct timeval _end_time;
};
