#ifndef __WIREMATRIX__
#define __WIREMATRIX__

#include <wirematrixentry.h>
#include <utility/matrix.h>

class Wire;
class WireMatrixIterator;


//! Die Schaltmatrix eines Routers.
/*!
 * Die Schaltmatrix eines Routers stellt Verbindungen zwischen ein- 
 * und ausgehenden Leitungen eines Routers dar (Eingaenge x Ausgaenge). 
 * Ein Eintrag verknuepft jeweils einen Eingang mit einem Ausgang. Mit 
 * Iteratoren koennen alle Eingaenge bzw. Ausgaenge zu einer Leitung 
 * untersucht und mit Manipulatoren modifiziert werden.
 * 
 * @see WireMatrixEntry
 * @see WireMatrixIterator
 * @see WireMatrixModifier
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class WireMatrix : public Matrix<WireMatrixEntry>
{

friend class WireMatrixEntry;
	
public:
	
	//! Struktur zur Verwaltung angelegter Pakete.
	/*!
	 * Ein Paket, dass von einem Router vermittelt wird, wird an dessen
	 * Schaltmatrix fuer den entsprechenden Eingang in Form dieser Struktur
	 * registriert.
	 */
	struct PacketInfo
	{
		//! Default-Konstruktor.
		PacketInfo() 
		: valid(false), number(0), packet(NULL), remain(NULL) {}
		
		//! Gueltigkeit.
		bool valid;

		//! Nummer des angelegten Paketes.
		unsigned int number;
		
		//! Angelegtes Paket.
		const Packet* packet;

		//! Verbleibende Ziele des angelegeten Paketes.
		Bitmask* remain;
	};

	//! Konstruktur.
	/*!
	 * Dieser Konstruktor wird in der Regel zum Zeitpunkt der
	 * Initialisierung eines Routers aufgerufen. Die Listen der
	 * Ein- und Ausgaenge sind dann leer.
	 * 
	 * @param in Die eingehenden Leitungen des zugehoerigen Routers.
	 * @param out Die ausgehenden Leitungen des zugehoerigen Routers. 
	 */
	WireMatrix(const std::vector<Wire*>& in, const std::vector<Wire*>& out)
	: Matrix<WireMatrixEntry>(in.size(), out.size()), inwires(in), outwires(out) {}

	//! Iterator ueber die verknuepften Ausgaenge zu einem Eingang.
	WireMatrixIterator* iterateOutWires(Wire* inwire);

	//! Iterator ueber die verknuepften Eingaenge zu einem Ausgang.
	WireMatrixIterator* iterateInWires(Wire* outwire);
		
	//! Hinzufuegen eines Ausganges.
	void newOutWire();

	//! Hinzufuegen eines Einganges.
	void newInWire();
		
	//! Zugriff auf Vektor aller verwalteten Eingaenge.
	const std::vector<Wire*>& getInWires() { return inwires; }
		
	//! Zugriff auf Vektor aller verwalteten Ausgaenge.
	const std::vector<Wire*>& getOutWires() { return outwires; } 

	//! Zugriff auf Informationen zu angelegten Paketen an Eingaengen.
	std::vector<PacketInfo>& managedPackets() { return m_packets; }
	
private:
		
	//! Referenz auf eingehende Leitungen des Routers
	const std::vector<Wire*>& inwires;

	//! Referenz auf ausgehende Leitungen des Routers
	const std::vector<Wire*>& outwires;

	//! Iteratoren ueber Eingaenge aus Sicht der Ausgaenge
	std::vector<WireMatrixIterator*> iw_iterators;
	
	//! Iteratoren ueber Ausgaenge aus Sicht der Eingaenge
	std::vector<WireMatrixIterator*> ow_iterators;

	//! Zu vermittelnde Pakete an Eingaengen
	std::vector<PacketInfo> m_packets;
};

inline std::ostream& operator<<(std::ostream& target, std::vector<WireMatrixEntry> toShow)
{
	if(toShow.size())
	{
		target << "{";
		
		for(unsigned int i=0;i<toShow.size()-1;i++)
			target << toShow[i] << ",";
		
		target << toShow[toShow.size()-1] << "}";
	}
	else
		target << "undefined";
	
	return target;
}

inline std::ostream& operator<<(std::ostream& target, WireMatrix& toShow)
{
	if(toShow.getYSize())
	{
		target << "{";
		
		for(int i=0;i<toShow.getYSize()-1;i++)
			target << toShow[i] << ",";
		
		target << toShow[toShow.getYSize()-1] << "}";
	}
	else
		target << "undefined";
	
	return target;
}

#endif

