#ifndef __WIREMATRIXENTRY__
#define __WIREMATRIXENTRY__

#include <bitset>
#include <vector>

class Bitmask;
class Wire;
class Packet;
class WireMatrix;

//! Bits zur Beschreibung von Schaltzustaenden.
enum e_bit_entries
{	
	O_BUSY,			// Ausgang belegt
	O_SCHEDULED,		// Scheduling durchgefuehrt
	O_FINISHED,		// Zustand fuer aktuellen Takt geklaert
		
	I_BUSY,			// Eingang belegt
	I_FINISHED,		// Zustand fuer aktuellen Takt geklaert				
	I_PACKET,		// Paket liegt an
		
	IO_VALID,		// Zulaessige Route
	IO_FIT,			// Backpressure-Bedingung erfllt
	IO_WON,			// Schaltung hergestellt
	IO_LOST,		// Schaltung abgelehnt
	IO_SWITCHED		// Schaltung hergestellt
};	

//! Eintraege einer Schaltmatrix.
/*!
 * Diese Klasse beschreibt die Eintraege einer Schaltmatrix eines 
 * Routers. Jeder Eintrag umfasst ein Bitfeld, zur Beschreibung des 
 * aktuellen Schaltzustandes und verschiedene Zugriffsmoeglichkeiten
 * auf beteiligte Leitungen und Paketinformationen.
 *
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class WireMatrixEntry : public std::bitset<12>
{
public:

	//! Konstruktor.
	/*!
	 * Der Konstruktor erzeugt einen Schaltmatrix-Eintrag zu gegebener
	 * Matrix, Zeile und Spalte. Default-Parameter sind aus Gruenden 
	 * der Definition notwendig (Matrix-Template) und sollten nicht 
	 * genutzt werden.
	 *
	 * @param wm Schaltmatrix fuer die Eintrag erzeugt werden soll.
	 * @param rowindex Die Zeile des Eintrages (eingehende Leitung).
	 * @param colindex Die Spalte des Eintrages (ausgehende Leitung).
	 */
	WireMatrixEntry(WireMatrix* wm=NULL, unsigned int rowindex=0, unsigned int colindex=0) 
	: std::bitset<12>(), _wm(wm), iw_index(rowindex), ow_index(colindex) {}

	//! Zugriff auf ausgehende Leitung des Routers.
	/*!
	 * Diese Methode liefert die ausgehende Leitung des zugehoerigen
	 * Routers, die von der Spaltenposition dieses Schaltmatrix-Eintrages 
	 * repraesentiert wird. 
	 * 
	 * @return Ausgehende Leitung zu diesem Eintrag,
	 */
	Wire* getOutWire() const;

	//! Zugriff auf eingehende Leitung des Routers.	
	/*!
	 * Diese Methode liefert die eingehende Leitung des zugehoerigen
	 * Routers, die von der Zeilenposition dieses Schaltmatrix-Eintrages 
	 * repraesentiert wird. 
	 * 
	 * @return Eingehende Leitung zu diesem Eintrag,
	 */
	Wire* getInWire() const;
	
	//! Zugriff auf momentan zu vermittelndes Paket.
	/*!
	 * Pakete werden von Routereingaengen zu Routerausgaengen vermittelt.
	 * Diese Methode liefert das momentan fuer die eingehende Leitung
	 * dieses Eintrages registrierte Paket.
	 *
	 * @return Momentan zu vermittelndes Paket zu diesem Eintrag.
	 */
	const Packet* currentPacket() const;

	//! Verbleibende Ziele des zu vermittelnden Paketes.
	/*!
	 * Werden die ein- und ausgehende Leitung eines Eintrages geschaltet,
	 * so sind die ueber den Ausgang erreichbaren Ziele abgearbeitet. Diese
	 * Methode liefert eine Bitmaske, die die noch offenen Ziele fuer
	 * das momentan zu vermittelnde Paket enthaelt. Ist diese Bitmaske
	 * leer, so wurde das Paket bereits vollstaendig vermittelt.
	 *
	 * @return Bitmaske mit offenen Zielen des zu vermittelnden Paketes 
	 * oder NULL, falls kein Paket auf eingehender Leitung bzw. registriert.
	 */
	Bitmask* remaining() const;

	//! Zulaessige Ziele ueber ausgehende Leitung.
	/*!
	 * Die Ziele, die ueber die ausgehende Leitung zu diesem Eintrag
	 * erreicht werden koennen unterscheiden sich u.U. bedingt durch
	 * Einschraenkungen des Routings von den zulaessigen Zielen. Diese
	 * Methode liefert die zulaessigen Ziele, die beim Vermitteln des
	 * Paketes als abgearbeit markiert werden koennen.
	 *
	 * @return Bitmaske mit zulaessigen Zielen ueber ausgendende Leitung.
	 */
	const Bitmask* targets() const;

private:

	//! Schaltmatrix zu diesem Eintrag.
	WireMatrix* _wm;
	
	//! Zeilenindex.
	unsigned int iw_index;
	
	//! Spaltenindex.
	unsigned int ow_index;
};

#endif
