#ifndef __XMLSIMULATION_H__
#define __XMLSIMULATION_H__

#include <xmlload.h>
#include <simulation.h>
#include <utility/hashmap.h>

class XMLElement;
class Configuration;


//! Klasse zur Initialisierung einer Simulation.
/*!
 * Diese Klasse stellt alle Methoden bereit, um aus einem XML-Dokument eine
 * CINSim-Simulation zu initialisieren.
 * 
 * @author Matthias Kuehm <kuehmi@cs.tu-berlin.de>
 */
class XmlSimulation : public Simulation
{
	
protected:
	
	//! Netzknoten. 
	typedef StringMap<XMLElement*> Connectables;
	
	//! Netzkanten.
	typedef StringMap<XMLElement*> Connectors;
		
	//! Einstellungen.
	typedef StringMap<DOMElement*> Settings;
	
	
	//! Implementierung der abstrakten Basisklasse XMLLoad.
	/*!
	 * Diese Methode wird nach dem Einlesen und Validieren einer XML-Datei
	 * aufgerufen und leitet die Initialisierung einer CINSim-Simulation ein.
	 */
	virtual void init();
  
	
	//! Netzkonfiguration parsen.
	/*!
	 * Eine Simulation beinhaltet mindestens eine Netzkonfiguration, die
	 * ein Verbindungsnetzwerk beschreibt. Im Falle von Rekonfiguration 
	 * koennen verschiedene Konfigurationen verknuepft werden, um die 
	 * zeitliche Veraenderung eines Verbindungsnetzwerkes zu beschreiben.
	 * Diese Methode erstellt rekursiv eine Netzkonfiguration anhand des
	 * zugehoerigen Top-Level XML-Elements. Der betrachtete Unterbaum 
	 * des DOM-Baumes wird bezueglich der eingebetteten Subnetze preorder 
	 * traversiert.
	 *
	 * @param top Element mit eingebettetem Subnetz.
	 * @param defaults Default-Werte fuer fehlende Attribute von Komponenten.
	 * @param config Die zu initialisierende Netzkonfiguration.
	 */
	void initConfig(const DOMElement* top, const Settings *defaults, Configuration *config);

	
	/*!
	 * Aktualisierung einer Hashmap mit Start- bzw. Zielelementen.
	 *
	 * Jedes Subnetz in einer Netzkonfiguration beinhaltet Knoten und Kanten.
	 * Start- und Zielknoten einer Kante werden dabei anhand von ID-Attributen
	 * definiert. Bei der Traversierung der Elemente eines Subnetztes werden
	 * zuerst alle Knoten erfasst und dann erst die Kanten. Knoten aus denen
	 * Instanzen der Klasse XMLElement erstellt werden, koennen in einer 
	 * Hashmap, mit dem ID-Attriut als Key, abgelegt werden, so dass beim Parsen 
	 * der Kanten die zu verbindenden Komponenten einer Konfiguration bestimmt
	 * werden koennen.
	 * 
	 * @param xmlelem Das erstellte XML-Element der Simulation.
	 * @param map Die zu aktualisierende Hashmap.
	 */
	void updateConnectables(XMLElement *xmlelem, Connectables *map) const;


	//! Aktualisierung der Einstellungen fuer Komponenten.
	/*!
	 * Die Subnetze einer Netzkonfiguration werden preorder traversiert. Auf
	 * jeder Ebene koennen ueber ein DOMElement Default-Einstellungen fuer
	 * die Komponenten einer Konfiguration festgelegt werden. Die in Form
	 * von Attributen gegebenen Einstellungen ersetzen fehlende Attribute 
	 * der einzelnen Komponenten und werden beim Traversieren der Subnetze 
	 * durchgereicht, so dass unvollstaendige Einstellungen fuer eine Netzebene
	 * durch die der uebergeordneten Ebene komplettiert werden.
	 *
	 * @param elem Element mit Einstellungen fuer Komponenten.
	 * @param current Die zu erstellende Hashmap der aktuellen Netzebene.
	 * @param previous Die Hashmap der uebergeordneten Netzebene.
	 */
	void updateSettings(DOMElement* elem, Settings *current, const Settings *previous=NULL);

private:

	//! Leitungen.
	/*
	 * Verbindungsleitungen einer Konfiguration koennen sich ueber mehrere
	 * Ebenen erstrecken und werden daher beim Parsen einer Konfiguration
	 * aufgesammelt.
	 */
	Connectors _wires;
};

#endif
